
TITLE ROM BIOS FOR MPX-16 SINGLE BOARD COMPUTER 
PAGE        60,132
;*
;*	date of this revision:	11/03/83
;*
;----------------------------
; EQUATES
;----------------------------
CODESG	    EQU	   0FE00H	; FOR 8K EPROM AT HIGH MEMORY
P8155_CMD   EQU	   01C0H	; 8155 COMMAND PORT
P8155_L     EQU    01C4H        ; 8155 LOW BYTE TIMER COUNT
P8155_H     EQU    01C5H        ; 8155 HIGH BYTE TIMER COUNT
P8255_A     EQU    01A0H        ; 8255 PORT A ADDRESS
P8255_B     EQU    01A1H        ; 8255 PORT B ADDRESS
P8255_C     EQU    01A2H        ; 8255 PORT C ADDRESS
P8255_D     EQU    01A3H        ; 8255 COMMAND PORT
P8259_M0    EQU    0140H        ; 8259 MASTER ADDRESS 0
P8259_M1    EQU    0141H        ; 8259 MASTER ADDRESS 1
P8259_S0    EQU    0120H        ; 8259 SLAVE ADDRESS 0
P8259_S1    EQU    0121H        ; 8259 SLAVE ADDRESS 1
P8253_0     EQU    01E0H        ; 8253 TIMER 1
P8253_1     EQU    01E1H        ; 8253 TIMER 2
P8253_2     EQU    01E2H        ; 8253 TIMER 3
P8253_CMD   EQU    01E3H        ; 8253 COMMAND PORT
P8251_CON   EQU    0161H        ; CON 8251 STATUS
P8251_AUX   EQU    0181H        ; AUX 8251 STATUS
P8237       EQU    0            ; 8237 BASE ADDRESS
P8272_STAT  EQU    20H          ; FDC STATUS REGISTER
P8272_DATA  EQU    21H          ; FDC DATA REGISTER
;
;	STATUS EQUATES FOR DISK I/O ROUTINES
;
TIME_OUT	EQU	080H	; ATTACHMENT FAILED TO RESPOND
BAD_SEEK	EQU	040H	; SEEK OPERATION FAILED
BAD_8272	EQU	020H	; FDC CONTROLLER CHIP FAILURE
BAD_CRC		EQU	010H	; CRC ERROR ON DISK READ
DMA_BOUNDARY	EQU	009H	; ATTEMPT TO DMA ACROSS 64K BOUNDARY
BAD_DMA		EQU	008H	; DMA OVERRUN ON OPERATION
RECORD_NOT_FND	EQU	004H	; REQUESTED SECTOR NOT FOUND
WRITE_PROTECT	EQU	003H	; WRITE ATTEMPTED ON PROTECTED DISK
BAD_ADDR_MARK	EQU	002H	; ADDRESS MARK NOT FOUND
BAD_CMD		EQU	001H	; BAD COMMAND SENT TO BIOS
PAGE_PORT	EQU	060H	; PAGE REGISTER FOR DMA, HI NIBBLE
MOTOR_PORT	EQU	0A0H	; MOTOR CONTROL, FDC RESET BIT
;
;	EXTERNAL PROCEDURES
;
EXTRN	PRINT_MESSAGE:FAR,KB_INIT:FAR,PRTSC_INT:FAR,ser_con_in:far
EXTRN	KB_IO:FAR,KB_INT_HDLR:FAR,VID_INT_HDLR:FAR,VID_PARM_TABLE:FAR
extrn	ser_con_out:far,msg_tbl:word
;
;
;----------------------------
; INTERRUPT LOCATIONS
;----------------------------
ABS0       SEGMENT  AT 0
;
EXTRN	INT_0_VEC:BYTE,NMI_VEC:WORD,PRTSC_VEC:WORD,IBM_INT:WORD
EXTRN	DISK_POINTER:DWORD,VID_PARM_PTR:DWORD,GR_CHAR_EXT:DWORD
EXTRN	MPX_INT:WORD
;
ABS0	ENDS

;----------------------------
; INITIALIZATION STACK
;----------------------------
STACK      SEGMENT  AT 030H
           DW       128 DUP(?)
TOP_STACK  LABEL    WORD
STACK      ENDS

;----------------------------
; RESERVED DATA AREAS
;----------------------------
DATA       SEGMENT  AT 040H
;
EXTRN	EQUIP_FLAG:WORD,MEM_SIZ:WORD,TIMER_LSW:WORD,TIMER_MSW:WORD
EXTRN	TIMER_FLG:BYTE,RECAL_FLG:BYTE,MOTOR_STAT:BYTE,MOTOR_CNT:BYTE
EXTRN	DISK_STAT:BYTE,DISK_STAT_ARRAY:BYTE,NUMSEC:BYTE
EXTRN	DISK_INT_FLG:BYTE,BREAK_FLAG:BYTE,REAL_TIME_FLAG:BYTE
EXTRN	PRT_BASE:WORD,SER_BASE:WORD,display_lines:byte
;
DATA         ENDS
;
;----------------------------
; ROM RESIDENT CODE   
;----------------------------
;
PUBLIC	START
;
CSEG	SEGMENT	COMMON

ASSUME    CS:CSEG,SS:STACK,ES:ABS0,DS:DATA

	ORG	01000H		; START AT FE00:1000
	DB	'MPX-16 BIOS COPR. MICROMINT 1983'
;
START	PROC
;
	CLI		; DISABLE INTERRUPTS
	CLD		; GO FORWARD FOR STRING OPS

;----------------------------
; SEGMENT REGISTER INIT
;----------------------------

	MOV	AX,STACK		; SETUP SS SEG REG
	MOV	SS,AX
	MOV	AX,OFFSET TOP_STACK ;SETUP SP REG
	MOV	SP,AX
	MOV	AX,DATA		; SETUP DS REG
	MOV	DS,AX
	MOV	AX,ABS0		; SETUP ES REG
	MOV	ES,AX

;----------------------------
; HARDWARE INITIALIZATION
;----------------------------

	MOV	DX,P8255_D	; 8255 COMMAND PORT
	MOV	AL,91H		; A + CL IN, B + CH OUT
	OUT	DX,AL		; INIT CHIP
	MOV	DX,P8255_C	; DISABLE PARITY CHECK
	MOV	AL,0F0H
	OUT	DX,AL		; PARITY DISABLED
	MOV	DX,P8155_CMD
	MOV	AL,040H
	OUT	DX,AL		; STOP RAM REFRESH TIMER
	MOV	DX,P8155_L
	MOV	AL,23H
	OUT	DX,AL		; LOAD LSB FOR TIMER
	MOV	DX,P8155_H
	MOV	AL,0C0H
	OUT	DX,AL		; LOAD MSB FOR TIMER
	MOV	DX,P8155_CMD
	MOV	AL,0C0H
	OUT	DX,AL		; START RAM REFRESH TIMER

; DMA AND TIMER INITIALIZATION FOR MEMORY REFRESH

	MOV	AL,0		; MASTER CLEAR OF 8237
	OUT	P8237+0DH,AL
	MOV	AL,24H		; DISABLE 8237A
	OUT	P8237+08H,AL
	MOV	AL,0		; SET ALL ADDRESSES TO 0
	OUT	P8237+0CH,AL	; SET F/F
	OUT	P8237+00H,AL
	OUT	P8237+00H,AL
	OUT	P8237+02H,AL
	OUT	P8237+02H,AL
	OUT	P8237+04H,AL
	OUT	P8237+04H,AL
	OUT	P8237+06H,AL
	OUT	P8237+06H,AL
	OUT	040H,AL		; CLEAR PAGE REGISTERS
	OUT	060H,AL
	OUT	080H,AL
	MOV	AL,0FFH		; SET ALL COUNTS HIGH
	OUT	P8237+01H,AL
	OUT	P8237+01H,AL
	OUT	P8237+03H,AL
	OUT	P8237+03H,AL
	OUT	P8237+05H,AL
	OUT	P8237+05H,AL
	OUT	P8237+07H,AL
	OUT	P8237+07H,AL

	MOV	AL,58H		; SET MODE FOR CHANNEL 0
	OUT	P8237+0BH,AL
	MOV	AL,41H		; SET MODE FOR CHANNEL 1
	OUT	P8237+0BH,AL
	INC	AL		; SET MODE FOR CHANNEL 2
	OUT	P8237+0BH,AL
	INC	AL		; SET MODE FOR CHANNEL 3
	OUT	P8237+0BH,AL
	MOV	AL,0		; ENABLE DMA
	OUT	P8237+08H,AL
	OUT	P8237+0AH,AL

; INTERRUPT CONTROLLER CHIPS (MASTER AND SLAVE) INITIALIZATION

; INITIALIZE MASTER

    MOV    DX,P8259_M0            ; M0-MASTER, PORT 0
    MOV    AL,011H                ; EDGE-TRIGGER, CASCADE MODE, ICW4 NEEDED
    OUT    DX,AL                  ; ICW1
    MOV    DX,P8259_M1            ; M1-MASTER, PORT 1
    MOV    AL,040H                ; INTERRUPTS 40H-47H                       
    OUT    DX,AL                  ; ICW2
    MOV    AL,002H                ; IR1 HAS A SLAVE                        
    OUT    DX,AL                  ; ICW3
    MOV    AL,00DH                ; BUFFERED MODE/MASTER, NORMAL EOI, 8088 MODE
    OUT    DX,AL                  ; ICW4
    MOV    AL,0FCH                ; MASK INTERRUPTS (ALL EXCEPT RTI, SLV)
    OUT    DX,AL                  ; OCW1

; INITIALIZE SLAVE

    MOV    DX,P8259_S0            ; S0-SLAVE, PORT 0
    MOV    AL,011H                ; EDGE-TRIGGER, CASCADE MODE, ICW4 NEEDED
    OUT    DX,AL                  ; ICW1
    MOV    DX,P8259_S1            ; S1-SLAVE, PORT 1
    MOV    AL,048H                ; INTERRUPTS 48H-4FH                       
    OUT    DX,AL                  ; ICW2
    MOV    AL,001H                ; SLAVE ID = 1                 
    OUT    DX,AL                  ; ICW3
    MOV    AL,009H                ; BUFFERED MODE/SLAVE, NORMAL EOI, 8088 MODE
    OUT    DX,AL                  ; ICW4
    MOV    AL,0DFH                ; MASK INTERRUPTS (ALL EXCEPT DISK)
    OUT    DX,AL                  ; OCW1

 ; INITIALIZE 8253 TIMER CHIP FOR REAL TIME INTERRUPT 

    MOV    DX,P8253_CMD           ; COMMAND PORT TO 8253
    MOV    AL,0B6H                ; CONTROL WORD
    OUT    DX,AL
    DEC    DX                     ; COUNT ADDRESS FOR CHANNEL 3
    MOV    AX,33746		  ; 36.4 INTERRUPTS/SEC (2*IBM)
    OUT    DX,AL                  ; LSB BYTE
    MOV    AL,AH
    OUT    DX,AL                  ; MSB BYTE

;-----------------------------
; SIZE MEMORY
;-----------------------------

    MOV    AX,1000H              ; ASSUME 64K
    MOV    ES,AX
    MOV    BX,64		; SIZE COUNTER
    MOV    SI,0
    MOV    DH,055H		; TEST PATTERN
MS1: MOV    ES:[SI],DH 		; WRITE TEST PATTERN
    MOV    DL,ES:[SI]
    CMP    DL,DH		; PATTERN STILL THERE?
    JNE    MS2
    ADD    BX,64		; ADD ANOTHER 64K TO SIZE COUNTER
    ADD    AX,1000H              ; NEXT 64K BLOCK
    CMP    AX,0B000H             ; 640K MAX
    JE     MS2
    MOV    ES,AX
    JMP    SHORT MS1
;*
;*	AT THIS POINT, BX CONTAINS THE NUMBER OF 1K SEGMENTS.  THIS
;*	WILL BE CONVERTED TO A NUMBER OF 64K SEGMENTS, AND ALL RAM
;*	LOCATIONS WILL THEN BE CLEARED TO 0.
;*
MS2:
	MOV	SI,BX		; SAVE THE SIZE COUNT
	MOV	CL,6		; DIVIDE BY 64 FOR 64K SEGMENTS
	SHR	BX,CL
	XOR	AX,AX		; START AT LOCATION 0
	MOV	DX,AX		; USE DX AS SEGMENT COUNTER
	MOV	ES,AX
;
CLEAR_MEM_LOOP:
;
	MOV	ES,DX		; NEXT SEGMENT
	mov	cx,08000h	; do 32k words at a time
	XOR	DI,DI		; START AT OFFSET 0
	rep	stosw
	ADD	DX,01000H
	DEC	BX
	JNE	CLEAR_MEM_LOOP
	MOV	MEM_SIZ,SI
;*
;*	FIRST WRITE A TRAP VECTOR FOR ALL POSSIBLE INTERRUPTS.
;*
	MOV	AX,ABS0		; BOTH SEGMENTS TO ABSOLUTE 0 SEGMENT
	MOV	ES,AX
	MOV	DS,AX
	MOV	DI,AX		; START AT 0
	MOV	AX,OFFSET TRAP_INT
	MOV	BX,CODESG
	MOV	CX,512		; 256 VECTORS * 2 WORDS
SET_TRAP:
	STOSW
	XCHG	AX,BX		; FIRST IP, THEN CS
	LOOP	SET_TRAP
	MOV	AX,DATA		; RESTORE BIOS DATA SEGMENT
	MOV	DS,AX
;
;-----------------------------
; INITIALIZE VECTOR TABLE
;-----------------------------

; INITIALIZE NMI INTERRUPT 

    MOV    AX,OFFSET NMI_INT      ; INIT OFFSET OF NMI SERVICE ROUTINE
    MOV    ES:NMI_VEC,AX
    MOV    AX,CODESG              ; INIT SEGMENT OF NMI SERVICE ROUTINE
    MOV    ES:NMI_VEC+2,AX

;
; INITIALIZE PRINT SCREEN INTERRUPT
;
    MOV    AX,OFFSET PRTSC_INT   ; PRINT SCREEN INTERRUPT VECTOR
    MOV    ES:PRTSC_VEC,AX
    MOV    AX,CODESG
    MOV    ES:PRTSC_VEC+2,AX
;
; INITIALIZE SOFT INTERRUPTS

    PUSH   DS
    MOV    AX,CODESG              ; INIT DS FOR TABLE TRANSFER
    MOV    DS,AX
    MOV    CX,IBM_TBL_SIZ         ; INIT COUNT FOR TRANSFER
    MOV    SI,OFFSET IBM_VEC_TBL  ; INIT SOURCE POINTER
    MOV    DI,OFFSET IBM_INT      ; INIT DESTINATION POINTER
    REP    MOVSW 

; INITIALIZE HARD INTERRUPTS

    MOV    CX,MPX_TBL_SIZ         ; INIT COUNT FOR TRANSFER
    MOV    SI,OFFSET MPX_VEC_TBL  ; INIT SOURCE POINTER
    MOV    DI,OFFSET MPX_INT      ; INIT DESTINATION POINTER
    CLD                           ; INIT DIRECTION
    REP    MOVSW 
    POP    DS                     ; RESTORE NORMAL DS

;*
;*	NOW TURN ON PARITY ERROR NMI INTERRUPTS
;*
	MOV	DX,P8255_C
	IN	AL,DX		; READ CURRENT BITS
	AND	AL,0C0H		; ENABLE NMI BITS
	OUT	DX,AL
;*
;*	INITIALIZE THE ON-BOARD SERIAL AND PRINTER BASE ADDRESSES.
;*
	MOV	SER_BASE,0161H	; CONSOLE CONTROL PORT
	MOV	SER_BASE+2,0181H ; AUX. SERIAL CONTROL PORT
	MOV	PRT_BASE,01A2H	; 8255A PORT C PRINTER CONTROL PORT
	mov	display_lines,24	; 25 lines default (0 base)
;
;-----------------------------
; READ SWITCHES AND SET PARAMS
;-----------------------------
	MOV	DX,P8255_A	; FIRST READ IN DIP SWITCHES
	IN	AL,DX
	AND	AL,3		; ONLY KEEP NUMBER OF DISK DRIVES
	MOV	CL,6
	SHL	AL,CL		; BITS 7&8 = DISK DRIVES
	mov	cl,0dh		; disk attached, 64k memory min
	or	cl,al		; save disk information
	IN	AL,DX		; READ SWITCHES AGAIN
	TEST	AL,010H		; TEST FOR TERMINAL OR KBD/MON
	JNE	TERM_MODE
;
	SHR	AL,1		; MOVE INTO CORRECT POSITIONS
	AND	AL,030H		; ONLY KEEP MONITOR INFORMATION
	mov	bh,2		; 2 default serial ports for equip_flag
	jmp	short set_mon
;
TERM_MODE:
	xor	ax,ax		; NO MONITORS ATTACHED
	mov	SER_BASE,0181h	; only use one serial port
	mov	ser_base+2,ax	; zero second port base address
	mov	bh,1		; 1 default serial port
;
set_mon:
	or	cl,al		; save monitor information

;*
;*	CHECK FOR IBM SERIAL AND PARALLEL DEVICES.  IF RESIDENT ISSUE
;*	AN INITIALIZATION CALL AND SET UP THE BIOS BASE ADDRESSES.
;*
	mov	bl,1		; 1 default printer port
	MOV	AH,0AAH		; PRINTER TEST PATTERN
	MOV	DI,2		; FIRST PRINTER IS THE ON-BOARD
	MOV	DX,03BCH	; TEST FIRST PRINTER PORT
	MOV	AL,AH		; GET TEST PATTERN
	OUT	DX,AL
	IN	AL,DX		; READ IT BACK
	CMP	AL,AH		; WAS THE CARD THERE ?
	JNE	CHECK_PRT2
	MOV	PRT_BASE[DI],DX	; STORE IT
	INC	BL
	ADD	DI,2		; BUMP POINTER FOR NEXT PRINTER
;
CHECK_PRT2:
	MOV	DX,0378H	; NEXT PORT ADDRESS
	MOV	AL,AH
	OUT	DX,AL
	IN	AL,DX
	CMP	AL,AH
	JNE	CHECK_PRT3
	MOV	PRT_BASE[DI],DX
	INC	BL
	ADD	DI,2		; BUMP POINTER AGAIN
;
CHECK_PRT3:
	MOV	DX,0278H	; NEXT PORT ADDRESS
	MOV	AL,AH
	OUT	DX,AL
	IN	AL,DX
	CMP	AL,AH
	JNE	SER1
	MOV	PRT_BASE[DI],DX
	INC	BL
;
SER1:	mov	ax,EQUIP_FLAG	; check for terminal mode
	mov	di,2		; default to only one serial port
	and	al,030h
	je	ser1b
	inc	di
	inc	di		; use both serial ports
ser1b:	MOV	DX,03FAH	; ADDRESS OF FIRST SERIAL CARD
	IN	AL,DX		; READ INTERRUPT ID REGISTER OF 8250
	TEST	AL,0F8H		; BITS 3-7 SHOULD BE 0
	JNZ	SER2
	MOV	SER_BASE[DI],DX
	ADD	DI,2
	INC	BH
;
SER2:	MOV	DX,02FAH	; NEXT ASYNC CARD ADDRESS
	IN	AL,DX
	TEST	AL,0F8H
	JNE	UPDATE_EQUIP
	MOV	SER_BASE[DI],DX
	INC	BH
;
UPDATE_EQUIP:
;
	CMP	BL,4		; ONLY 2 BITS FOR # PRINTERS
	JNE	UE1
	DEC	BL		; ONLY USE FIRST 3
UE1:	ROR	BL,1
	ROR	BL,1		; MOVE PRINTER BITS TO BITS 6,7
	SHL	BH,1		; MOVE SERIAL BITS TO BITS 1,2,3
	XOR	AH,AH		; START WITH 0
	OR	AH,BH		; ADD IN # SERIAL CHANNELS
	OR	AH,BL		; ADD IN # PRINTER CHANNELS
;*
;*	NOW CHECK IF THE GAME ADAPTER IS INSTALLED
;*
	MOV	DX,0201H	; GAME CARD ADDRESS
	IN	AL,DX
	TEST	AL,0F0H
	JNZ	SAVE_EQUIP
	OR	AH,010H		; SET THE GAME CARD BIT
;
SAVE_EQUIP:
	MOV	CH,AH		; GET THE HIGH BYTE
	MOV	EQUIP_FLAG,CX	; SAVE NEW SYSTEM STATUS
;*
;*	Initialize the on-board serial port(s).  Note that if the system
;*	is using the console serial port for primary communications (if
;*	sw1, pos 5 is off), then only the aux port defaults to a system
;*	serial port.
;*
	test	cl, 030h	; terminal mode ?
	je	get_sw_baud
	mov	al,0eh		; no, default to 9600 baud
	jmp	short init_ob_ser
;
get_sw_baud:
	mov	dx,P8255_A	; yes, read in baudrate from switches
	in	al,dx
	and	al,0eh		; only use baudrate switches
;
init_ob_ser:
	or	al,2		; use 7 data bits
	xor	dx,dx		; serial channel 0
	xor	ah,ah		; initialize code
	int	014h		; use bios call
;
	test	cl,030h		; if in terminal mode only do one port
	je	init_console
;
	inc	dl		; serial channel 1
	xor	ah,ah		; initialize code
	mov	al,082h		; 1200 baud, no parity, 1 stop, 7 data
	int	014h
;*
;*	Now determine which display mode to use from the dip-switch
;*	settings.
;*
init_console:
	and	cl,030h		; strip off other bits
	xor	ax,ax		; start with a 0 code
	cmp	cl,0		; terminal mode ?
	je	init_conout
	cmp	cl,030h		; monochrome monitor, or both ?
	je	init_conout
	inc	al
	cmp	cl,010h		; 40x25 mode ?
	je	init_conout
	mov	al,3		; 80x25 mode
;
;*
;*	Note that the int 010h routine will check equip_flag to
;*	determine if output should be to the monitor adapter card or to
;*	the on-board serial port.
;*
init_conout:
	int	010h		; ah=0 here
	mov	ax,CODESG
	mov	ds,ax		; use rom-based message table
	mov	si,offset msg_tbl
	xor	ax,ax		; MPX-16 Loader message
	call	PRINT_MESSAGE	; print the message on the output device
	call	KB_INIT		; Now initialize the input device
;
;-----------------------------
; BEGIN BOOTSTRAP
;-----------------------------

	INT    19H		; XFER TO BOOT ROUTINE
;
;	Dummy returns - user is responsible for setting up the vector
;	for the appropriate routine.
;
IBM_CASSETTE:			; INT 015H
IBM_BASIC:			; INT 018H
IBM_KB_BREAK:			; INT 01BH
IBM_TIME_BREAK:			; INT 01CH
IBM_VID_EXT:			; INT 01FH
	iret			; just return
;
;-------------------------------
; NMI INTERRUPT HANDLER
;-------------------------------
;
NMI_INT    PROC    NEAR
	PUSH   AX
	push   ds
	push   si
	mov    ax,CODESG
	mov    ds,ax
	mov    si,offset msg_tbl
	MOV    AL,3		; PARITY ERROR MESSAGE
	CALL   PRINT_MESSAGE
	pop    si
	pop    ds
	POP    AX
	IRET
NMI_INT        ENDP
;*
;*	UNEXPECTED TRAP INTERRUPT HANDLER
;*
TRAP_INT PROC	NEAR
	PUSH	AX
	push	ds
	push	si
	mov	ax,CODESG
	mov	ds,ax
	mov	si,offset msg_tbl	; rom-based messages
	MOV	AL,4			; unxpected interrupt msg
	CALL	PRINT_MESSAGE
	pop	si
	pop	ds
	POP	AX
	IRET
TRAP_INT ENDP
;
;------------------------------------
; REAL-TIME CLOCK INTERRUPT
;------------------------------------
;
REAL_TIME       PROC     FAR
        STI
        PUSH    DS
        PUSH    DX
        PUSH    AX
        MOV     AX,DATA
        MOV     DS,AX             ; LOAD DATA SEGMENT
	MOV	AL,REAL_TIME_FLAG
	INC	AL
	MOV	REAL_TIME_FLAG,AL
	TEST	AL,1
	JNE	RT3
        DEC     MOTOR_CNT         ; TIMEOUT FOR DRIVE MOTORS
        JNZ     RT1
        XOR     AX,AX
        MOV     MOTOR_STAT,AL     ; MOTOR ON BITS OFF
        OUT     MOTOR_PORT,AL     ; ACTUALLY TURN OFF MOTORS
RT1:    INC     TIMER_LSW
        JNZ     CHECK_DAY
        INC     TIMER_MSW
CHECK_DAY:
	CMP	TIMER_MSW,24	; 24 HOURS ?
	JNE	RT2
	CMP	TIMER_LSW,160	; REALLY 24 HOURS ?
	JNE	RT2
	MOV	TIMER_LSW,0
	MOV	TIMER_MSW,0	; START A NEW DAY AT 0
        MOV     TIMER_FLG,1	; FLAG THE TIMER OVERFLOW
RT2:	INT	01CH		; USER TIMER BREAK
RT3:    MOV     DX,P8259_M0
        MOV     AL,060H
        OUT     DX,AL             ; RESET MASTER 8259
        POP     AX
        POP     DX
        POP     DS
        IRET
REAL_TIME       ENDP

;--INT 19 ------------------------
;BOOT STRAP LOADER
;    TRACK 0, SECTOR 1 OF DRIVE 0 IS READ INTO MEMORY AT
;    0000:7C00H AND CONTROL IS TRANSFERRED THERE.
;
;    IF A SUCCESSFUL READ IS NOT
;    ACCOMPLISHED, THEN CONTROL IS TRANSFERRED TO THE MONITOR
;    PROM AT FC00:0000H.
;
;---------------------------------

BOOT_STRAP      PROC      FAR     ; IBM COMPAT BOOTSTRAP     (INT 19)

    STI
    XOR    SI,SI	; RETRY COUNTER
;
INIT_LOOP:
;
    XOR    AH,AH	; INIT FUNCTION
    INT    13H		; INIT FDC
    JNC    READ_SEC
    INC    SI		; FOUR RETRIES?
    CMP    SI,4
    JAE    BS1
    JMP    SHORT INIT_LOOP
;
READ_SEC:
    XOR    SI,SI	; RETRY COUNTER
;
READ_LOOP:
    MOV    AL,1
    MOV    AH,2
    MOV    BX,0
    MOV    ES,BX
    MOV    BX,7C00H
    MOV    CX,1
    MOV    DX,0
    INT    13H		; READ IN SECTOR 0, TRACK 0
    JNC     GOTO_DOS
    INC    SI		; FOUR RETRIES?
    CMP    SI,4
    JAE    BS1
    JMP    SHORT READ_LOOP
;
GOTO_DOS:
;
    MOV    AX,0
    PUSH   AX
    MOV    AX,7C00H
    PUSH   AX
    RET

BS1: mov   si,offset msg_tbl
    mov    ax,CODESG
    mov    ds,ax
    MOV    AL,2
    CALL   PRINT_MESSAGE
    MOV    AX,0FC00H              ; ERROR, GO TO MONITOR
    PUSH   AX
    MOV    AX,0
    PUSH   AX
    RET

BOOT_STRAP    ENDP

;--INT 1A--
;
;  TIME OF DAY
;
;  AH=0 MEANS READ, AH=1 MEANS WRITE
;
;  CURRENT CLOCK READS IN 50MS UNITS (NON IBM)
;
;----------

TIME_DAY       PROC    FAR
        STI
        PUSH  DS
        PUSH  AX
        MOV   AX,DATA
        MOV   DS,AX
        POP   AX
        OR    AH,AH
        JNZ   TD1
        CLI
        MOV   AL,TIMER_FLG
        MOV   TIMER_FLG,0
        MOV   CX,TIMER_MSW
        MOV   DX,TIMER_LSW
        JMP   SHORT TD2
TD1:    CLI
        MOV   TIMER_FLG,0
        MOV   TIMER_MSW,CX
        MOV   TIMER_LSW,DX
TD2:    POP   DS
        IRET
TIME_DAY      ENDP

;--INT 11--
;
;  EQUIPMENT FLGS
;
;----------

IBM_EQUIP     PROC       FAR
        STI
	PUSH	DS
	MOV	AX,DATA
	MOV	DS,AX
        MOV      AX,EQUIP_FLAG
	POP	DS
        IRET
IBM_EQUIP     ENDP

;--INT 12--
;
;  MEMORY CALL
;
;----------

IBM_MEMORY    PROC        FAR
        STI
        PUSH    DS
        MOV     AX,DATA
        MOV     DS,AX
        MOV     AX,MEM_SIZ
        POP     DS
        IRET
IBM_MEMORY    ENDP

;--INT 14-------------------------
;
;  SERIAL I/O
;
;  BYTE I/O VIA BOTH CONSOLE AND AUX 8251S
;
;  (DX) = THE RS-232C CHANNEL NUMBER
;	THE BASE ADDRESSES OF UP TO 4 CHANNELS CAN BE STORED IN THE
;	BIOS DATA AREA STARTING AT 40:0.  POWER-ON DEFAULTS ARE FOR
;	CHANNELS 0 AND 1 TO BE THE ON-BOARD SERIAL PORTS.
;
;  (AH) = 0/INIT CHIP
;
;    (AL) = INIT PARAMETERS
;
;    7   6   5       4  3         2          1  0
;    BAUD RATE     -PARITY--   STOPBITS   CHAR LENGTH
;
;    000 - 110     X0 - NONE    0 - 1     10 - 7 BITS
;    001 - 150     01 - ODD     1 - 2     11 - 8 BITS
;    010 - 300     11 - EVEN
;    011 - 600
;    100 - 1200
;    101 - 2400
;    110 - 4800
;    111 - 9600
;
;  (AH) = 1/SEND CHAR
;
;    (AL) = CHAR TO BE SENT
;
;  (AH) = 2/RECV CHAR
;
;    (AL) = WILL CONTAIN CHAR RECVD
;
;  (AH) = 3/GET STATUS
;
;    (AL) = WILL CONTAIN STATUS BITS
;
;---------------------------------

SER_IO        PROC    FAR

    STI                           ; ENABLE INTERRUPTS
    PUSH   DS
    PUSH   BX                     ; SAVE REGS
    PUSH   CX
    PUSH   DX
    PUSH   SI
    MOV    CX,DATA
    MOV    DS,CX
    MOV    SI,DX		; GET BASE PORT ADDRESS
    SHL    SI,1			; WORD OFFSETS
    MOV    DX,SER_BASE[SI]
    OR     DX,DX                  ; GET PORT SELECTION
    JZ     SER_IO_RET
    OR     AH,AH                  ; TEST WHICH FUNCTION
    JZ     INIT_SER_IO
    DEC    AH
    JNZ    S1
    JMP    SEND_SER_IO
S1: DEC    AH
    JNZ    S2
    JMP    RECV_SER_IO
S2: DEC    AH
    JZ     STAT_SER_IO

SER_IO_RET:

    POP    SI                     ; RESTORE REGS
    POP    DX
    POP    CX
    POP    BX
    POP    DS
    IRET

INIT_SER_IO:

    TEST   DH,2			; ON-BOARD SERIAL IF 0
    JZ     ONBD_SER_INIT
    JMP    EXP_SER_INIT
;
ONBD_SER_INIT:

    PUSH   AX			  ; SAVE THE PARAMETERS FOR LATER
    MOV    CL,AL		  ; ALSO STORE IN CL
    SUB    AL,AL                  ; RESET 8251
    OUT    DX,AL
    OUT    DX,AL
    OUT    DX,AL
    MOV    AL,40H
    OUT    DX,AL
    MOV    AL,CL                  ; CONSTRUCT MODE BYTE
    AND    AL,18H                 ; ISOLATE PARITY BITS
    SHL    AL,1                   ; SHIFT INTO PLACE
    OR     AL,42H                 ; SET 16X BAUD FACTOR, 1 STOP BIT
    TEST   CL,4                   ; 2 STOP BITS?
    JZ     S3                     ; NO
    OR     AL,80H                 ; SET 2 STOP BITS
S3: MOV    AH,CL                  ; SAVE INTERMEDIATE RESULTS
    AND    AH,3                   ; ISOLATE WORD LEN BITS
    SHL    AH,1                   ; SHIFT INTO PLACE
    SHL    AH,1
    OR     AL,AH                  ; XFR BITS TO MODE BYTE
    OUT    DX,AL                  ; OUTPUT MODE BYTE
    MOV    AL,037H                ; ENABLE COMMUNICATION
    OUT    DX,AL
    MOV    AL,CL                  ; GET BAUD RATE INFO
    SHR    AL,1                   ; MAKE WORD INDEX
    SHR    AL,1
    SHR    AL,1
    SHR    AL,1
    AND    AX,0EH                 ; CLEAN UP INDEX
    MOV    BX,OFFSET BAUD_RATE    ; GET TABLE ADDR
    ADD    BX,AX                  ; POINT TO TABLE VALUE
    MOV    BX,CS:[BX]             ; GET COUNTER VALUE
    MOV    CX,P8253_0             ; LOAD DEFAULT TIMER CHANNEL (CONSOLE)
    MOV    AL,36H                 ; LOAD DEFAULT COMMAND
    CMP    DX,P8251_CON           ; CONSOLE?
    JE     S4                     ; YES
    MOV    CX,P8253_1             ; LOAD AUX TIMER CHANNEL
    MOV    AL,76H                 ; LOAD AUX COMMAND
S4: MOV    DX,P8253_CMD           ; COMMAND PORT
    OUT    DX,AL                  ; SEND TO 8253
    MOV    DX,CX                  ; GET 8253 CHANNEL ADDR
    MOV    AL,BL                  ; GET LSB
    OUT    DX,AL
    MOV    AL,BH                  ; GET MSB
    OUT    DX,AL
    CMP    DX,P8253_0             ; CONSOLE ADDR?
    MOV    DX,P8251_CON           ; RESTORE CON ADDR
    JE     RESTORE_PARMS          ; DEFAULT WAS CORRECT
    MOV    DX,P8251_AUX           ; RESTORE AUX ADDR
RESTORE_PARMS:
    POP    AX
    JMP    SHORT ONBD_SER_STAT	  ; RETURN STATUS IN AH

STAT_SER_IO:

    TEST   DH,2			  ; ON-BOARD SERIAL IF A 0
    JZ     ONBD_SER_STAT
    JMP    EXP_SER_STAT
;
ONBD_SER_STAT:

    MOV    CL,AL		  ; SAVE AL VALUE
    IN     AL,DX                  ; GET STATUS
    CALL   FORMAT_SER_STAT
    MOV    AL,CL		  ; RESTORE THE ORIGINAL AL VALUE
    JMP    SER_IO_RET             ; RETURN

SEND_SER_IO:

    TEST   DH,2			; ON-BOARD SERIAL IF A 0
    JZ     ONBD_SER_XMIT
    JMP    EXP_SER_XMIT
;
ONBD_SER_XMIT:

    XOR    CX,CX		  ; TIMEOUT COUNTER
    MOV    BL,AL                  ; SAVE CHAR
S5: IN     AL,DX                  ; GET STATUS
    TEST   AL,1                   ; READY?
    JNZ    XMIT_ONBD_CHAR
    LOOP   S5
;
ONBD_SER_TIMEOUT:

    OR     AH,080H		  ; SET THE TIMEOUT FLAG
    MOV    AL,BL		; RESTORE THE CHARACTER
    JMP    SER_IO_RET
;
XMIT_ONBD_CHAR:

    MOV    AL,BL                  ; GET CHAR
    DEC    DX                     ; POINT TO DATA REG
    OUT    DX,AL                  ; SEND CHAR
    JMP    SER_IO_RET             ; RETURN

RECV_SER_IO:

    TEST   DH,2			; ON-BOARD SERIAL IF A 0
    JZ     ONBD_SER_RCV
    JMP    EXP_SER_RCV
;
ONBD_SER_RCV:
;
    AND    BREAK_FLAG,07FH        ; TURN OFF BIOS BREAK FLAG
S6: IN     AL,DX                  ; GET STATUS
    TEST   AL,2                   ; CHAR READY?
    JNZ    RECV_CHAR
    TEST   BREAK_FLAG,080H        ; USER BREAK ?
    JZ     S6
    JMP    SHORT ONBD_SER_TIMEOUT
;
RECV_CHAR:
;
    DEC    DX                     ; POINT TO DATA REG
    IN     AL,DX                  ; GET CHAR
    MOV    CL,AL		  ; SAVE THE CHARACTER
    INC    DX			  ; AND THEN READ NEW STATUS BYTE
    IN     AL,DX
    CALL   FORMAT_SER_STAT	  ; PUT IN CORRECT BIT POSITIONS
    AND    AH,01EH		  ; ONLY RETURN ERROR BITS
    MOV    AL,CL		  ; AND THEN GET THE CHARACTER BACK
    JMP    SER_IO_RET
;
EXP_SER_INIT:
;
 	MOV	CH,AL		; SAVE PARAMETERS
	MOV	BX,AX
	MOV	AL,080H		; DLAB BIT FOR 8250
	ADD	DX,3		; 8250 LINE CONTROL REGISTER
	OUT	DX,AL
	MOV	CL,4
	SHR	BX,CL		; USE BAUDRATE BITS AS A WORD OFFSET
	AND	BX,0EH		; INTO A TABLE
	ADD	BX,OFFSET EXP_BAUDRATES
	MOV	AX,CS:[BX]
	XCHG	AH,AL		; DO MSB FIRST
	DEC	DX
	DEC	DX		; DIVISOR LATCH MSB
	OUT	DX,AL
	DEC	DX
	XCHG	AH,AL		; NOW DO DIVISOR LATCH LSB
	OUT	DX,AL
	ADD	DX,3		; BACK TO LINE CONTROL REGISTER
	MOV	AL,CH		; GET THE PARAMETERS BACK
	AND	AL,01FH		; STRIP OFF BAUD RATE INFO
	OUT	DX,AL
	DEC	DX
	DEC	DX		; 8250 INTERRUPT ENABLE REGISTER
	XOR	AL,AL
	OUT	DX,AL
;
EXP_SER_STAT:
;
	MOV	DX,SER_BASE[SI]	; ENSURE CORRECT PORT ADDRESS
	ADD	DX,5		; 8250 LINE CONTROL STATUS REGISTER
	IN	AL,DX
	MOV	AH,AL		; RETURN THIS IN HIGH BYTE
	INC	DX		; MODEM CONTROL STATUS
	IN	AL,DX
	JMP	SER_IO_RET
;
EXP_SER_XMIT:
;
	MOV	BL,AL		; SAVE THE CHARACTER FOR LATER
	ADD	DX,4		; 8250 MODEM CONTROL
	MOV	AL,3		; DTR AND RTS BITS
	OUT	DX,AL
	INC	DX
	INC	DX		; MODEM STATUS REGISTER
	XOR	CX,CX		; TIMEOUT COUNTER
;
XMIT_DSR_LOOP:
;
	IN	AL,DX
	TEST	AL,020H		; DSR ?
	JNZ	CHECK_CTS
	LOOP	XMIT_DSR_LOOP
	JMP	SHORT XMIT_TIMEOUT
;
CHECK_CTS:
;
	XOR	CX,CX		; TIMEOUT COUNTER
;
CTS_LOOP:
;
	IN	AL,DX
	TEST	AL,010H		; CTS ?
	JNZ	CHECK_TXRDY
	LOOP	CTS_LOOP
	JMP	SHORT XMIT_TIMEOUT
;
CHECK_TXRDY:
;
	DEC	DX		; 8250 LINE STATUS REGISTER
	XOR	CX,CX		; TIMEOUT COUNTER
;
TXRDY_LOOP:
;
	IN	AL,DX
	TEST	AL,020H		; TRANSMITTER READY ?
	JNZ	XMIT_EXP_CHAR
	LOOP	TXRDY_LOOP
;
XMIT_TIMEOUT:
;
	MOV	AL,BL		; RESTORE THE CHARACTER
	OR	AH,080H		; SET THE TIMEOUT BIT
	JMP	SER_IO_RET
;
XMIT_EXP_CHAR:
;
	SUB	DX,5		; 8250 DATA PORT
	MOV	AL,BL		; RESTORE CHARACTER
	OUT	DX,AL
	JMP	SER_IO_RET
;
EXP_SER_RCV:
;
	ADD	DX,4		; 8250 MODEM CONTROL REGISTER
	MOV	AL,1		; DTR BIT
	OUT	DX,AL
	INC	DX
	INC	DX		; MODEM STATUS REGISTER
	AND	BREAK_FLAG,07FH	; TURN OFF THE BREAK BIT
	XOR	CX,CX		; TIMEOUT COUNTER
;
RCV_DSR_LOOP:
;
	IN	AL,DX
	TEST	AL,020H		; DSR ?
	JNZ	CHECK_RXRDY
	LOOP	RCV_DSR_LOOP
	JMP	SHORT RCV_EXP_TIMEOUT
;
CHECK_RXRDY:
;
	DEC	DX		; 8250 LINE STATUS REGISTER
;
RXRDY_LOOP:
;
	IN	AL,DX
	TEST	AL,1
	JNZ	READ_SER_CHAR
	TEST	BREAK_FLAG,080H	; USER BREAK REQUESTED ?
	JZ	RXRDY_LOOP
;
RCV_EXP_TIMEOUT:
;
	OR	AH,080H		; SET THE TIMEOUT FLAG
	JMP	SER_IO_RET
;
READ_SER_CHAR:
;
	AND	AL,01EH		; ONLY RETURN THE ERROR BITS
	MOV	AH,AL		; PUT IN HIGH BYTE
	SUB	DX,5		; BACK TO 8250 DATA PORT
	IN	AL,DX
	JMP	SER_IO_RET
;
SER_IO     ENDP

FORMAT_SER_STAT	PROC	NEAR
;
;*	PUT THE 8251 STATUS BITS INTO THE CORRECT BIT POSITIONS
;*	FOR MAXIMUM SOFTWARE COMPATIBILITY.
;*	ON ENTRY, AL CONTAINS THE 8251 STATUS INFORMATION.
;*	ON EXIT, AH CONTAINS THE CORRECTED STATUS BITS.
;*
	MOV	BL,AL		; SAVE STATUS FOR LATER USE
	SHR	AL,1
	MOV	AH,AL
	AND	AH,5		; RX DATA RDY, PARITY CORRECT
	SHR	AL,1
	MOV	BH,AL		; SAVE IT
	AND	AL,018H
	OR	AH,AL		; FRAMING ERROR, BREAK DET. CORRECT
	MOV	AL,BH
	AND	AL,020H		; DSR BIT
	SHR	BH,1
	AND	BH,2
	OR	AH,BH		; OVERRUN ERROR OK
	AND	BL,4		; NOW DO THE TXEMPTY BIT
	SHL	BL,1
	SHL	BL,1
	SHL	BL,1
	OR	AH,BL		; TX HOLDING REGISTER EMPTY FLAG
	SHL	BL,1
	OR	AH,BL		; TX EMPTY FLAG
	RET
;
FORMAT_SER_STAT	ENDP
;
;--INT 17 ------------------------
;
;  PRINTER I/O
;
;  (DX) - CONTAINS THE PRINTER CHANNEL NUMBER (0 TO 3).  THE ACTUAL
;	PORT ADDRESSES ARE STORED IN THE BIOS DATA AREA STARTING AT
;	40:8H.  CHANNEL 0 DEFAULTS TO THE ON-BOARD PRINTER PORT.
;
;  (AH) = 0/SEND CHAR
;    (AL) CONTAINS THE CHAR TO BE SENT
;
;  (AH) = 1/SEND INIT TO PRINTER
;
;  (AH) = 2/GET STATUS
;    (AH) WILL CONTAIN THE STATUS
;	BIT 0 = TIME OUT
;	BIT 1,2 = NOT USED
;	BIT 3 = I/O ERROR
;	BIT 4 = PRINTER SELECTED
;	BIT 5 = OUT OF PAPER
;	BIT 6 = PRINTER ACKNOWLEDGE
;	BIT 7 = PRINTER BUSY
;
;		NOTE:  ON-BOARD PRINTER USES ONLY BITS 0,4,5,7
;
;----------------------------------

PRINT_IO    PROC    FAR

    STI                           ; ENABLE INTERRUPTS
    PUSH    BX
    PUSH   CX
    PUSH   DX
    PUSH   SI
    PUSH   DS
    MOV    CX,DATA
    MOV    DS,CX
    MOV    SI,DX		; DETERMINE BASE ADDRESS
    SHL    SI,1			; WORD OFFSETS INTO TABLE
    MOV    DX,PRT_BASE[SI]
    OR     DX,DX		; NO PRINTER CARD IF 0
    JZ     PRINT_IO_RET
    OR     AH,AH                  ; TEST WHICH FUNCTION
    JZ     SEND_PRINT_IO
    DEC    AH
    JZ     INIT_PRINT_IO
    DEC    AH
    JNZ    PRINT_IO_RET
    TEST   DH,2			; ON-BOARD PRINTER IF A 0
    JZ     ONBD_PRT_STAT
;
EXP_PRT_STAT:
;
	MOV	CL,AL		; SAVE AL REGISTER FOR LATER
	INC	DX		; POINT TO PRINTER STATUS REGISTER
	IN	AL,DX		; GET THE STATUS
	MOV	AH,AL		; RETURN RESULT IN AH REGISTER
	AND	AH,0F8H		; BITS 1 AND 2 NOT USED, CLR TIMEOUT
	XOR	AH,048H		; CHANGE POLARITY OF BITS 3 AND 7
	MOV	AL,CL		; RESTORE THE AL REGISTER
	JMP	SHORT PRINT_IO_RET
;
ONBD_PRT_STAT:
;
    CALL   STAT_PRINT_IO

PRINT_IO_RET:

    POP    DS
    POP    SI
    POP    DX                     ; RESTORE REGS
    POP    CX
    POP    BX
    IRET

INIT_PRINT_IO:

    TEST   DH,2			; ON-BOARD PRINTER IF A 0
    JZ     ONBD_PRT_INIT
    JMP    SHORT EXP_PRT_INIT
;
ONBD_PRT_INIT:
;
    XCHG   AH,AL                  ; SAVE AL
    MOV    AL,0EH                 ; SEND INPUT PRIME* LOW (RESET)
    INC    DX                     ; USE CONTROL PORT SET/RESET FEATURE
    OUT    DX,AL                  ; RESET PRINTER
    MOV    CX,1000H               ; DELAY
P1: LOOP   P1
    MOV    AL,0FH                 ; INPUT PRIME* HIGH
    OUT    DX,AL
    DEC    DX                     ; POINT TO STATUS LINES
    XCHG   AH,AL                  ; RESTORE AL
    CALL   STAT_PRINT_IO          ; GO GET STATUS
    JMP    SHORT PRINT_IO_RET

SEND_PRINT_IO:

    TEST   DH,2			; ON-BOARD PRINTER IF A 0
    JZ     ONBD_PRINT_OUT
    JMP    EXP_PRT_OUT
;
ONBD_PRINT_OUT:
;
    MOV    AH,AL                  ; SAVE CHAR
    DEC    DX                     ; DATA PORT B
    OUT    DX,AL                  ; SETUP DATA LINES
    INC    DX                     ; STATUS PORT C
    MOV    BL,11		; NUMBER OF TIMEOUTS
    SUB    CX,CX                  ; TIMEOUT VALUE
P2: IN     AL,DX                  ; GET STATUS
    TEST   AL,8                   ; BUSY?
    JNZ    P3                     ; NO
    LOOP   P2                     ; TRY AGAIN
    DEC    BL
    JNZ    P2			; TRY AGAIN
    MOV    AL,AH		; FIRST RESTORE THE AL REGISTER
    CALL   STAT_PRINT_IO
    OR     AH,1			; SET THE TIMEOUT BIT
    JMP    SHORT PRINT_IO_RET           ; RETURN
P3: mov    al,ah		; restore al
    call    STAT_PRINT_IO
    push    ax			; save status
    INC    DX                     ; USE CNTRL BIT SET/RESET FEATURE
    MOV    AL,0CH                 ; STROBE LOW
    OUT    DX,AL
    MOV    AL,0DH                 ; STROBE HIGH
    OUT    DX,AL
    DEC    DX                     ; POINT TO STATUS LINES
    pop    ax			; restore status and data
    JMP    SHORT PRINT_IO_RET
;
EXP_PRT_INIT:
;
	MOV	BL,AL		; SAVE THE AL REGISTER
	MOV	AL,8		; DRIVE THE INIT LINE TO LOW
	INC	DX
	INC	DX		; PRINTER OUTPUT PORT
	OUT	DX,AL
	MOV	CX,1000
;
PRT_INIT_LOOP:
;
	LOOP	PRT_INIT_LOOP	; WAIT A WHILE !
	MOV	AL,0CH		; INIT HIGH, NO AUTO LF, INTERRUPTS OFF
	OUT	DX,AL
	MOV	AL,BL		; RESTORE THE AL REGISTER
	DEC	DX
	DEC	DX		; BACK TO BASE ADDRESS
	JMP	EXP_PRT_STAT
;
EXP_PRT_OUT:
;
	MOV	BL,AL		; SAVE THE CHARACTER
	MOV	BH,11		; OUTER TIMEOUT LOOP COUNTER
	XOR	CX,CX		; INNER TIMEOUT LOOP COUNTER
	OUT	DX,AL		; FIRST WRITE THE CHARACTER
	INC	DX		; PRINTER STATUS PORT ADDRESS
;
PRT_BUSY_LOOP:
;
	IN	AL,DX
	MOV	AH,AL		; SAVE STATUS BITS
	TEST	AL,080H		; PRINTER BUSY ?
	JNZ	STB_THE_CHAR
	LOOP	PRT_BUSY_LOOP	; INNER TIMEOUT LOOP
	DEC	BH
	JNZ	PRT_BUSY_LOOP	; OUTER TIMEOUT LOOP
	OR	AH,1		; SET THE TIMEOUT BIT
	AND	AH,0F9H		; BITS 1 AND 2 ARE NOT USED
	XOR	AH,048H		; INVERT BITS 3 AND 7
	MOV	AL,BL		; RESTORE THE CHARACTER IN AL
	JMP	PRINT_IO_RET
;
STB_THE_CHAR:
;
	MOV	AL,0DH		; SET THE STROBE BIT HIGH TO START
	INC	DX		; POINT TO PRINTER CONTROL PORT
	OUT	DX,AL
	DEC	AL		; STROBE BIT TO LOW
	OUT	DX,AL
	MOV	AL,BL		; RESTORE THE CHARACTER IN AL
	DEC	DX
	DEC	DX		; BACK TO BASE ADDRESS
	JMP	EXP_PRT_STAT	; NOW RETURN THE STATUS
PRINT_IO   ENDP
;
STAT_PRINT_IO  PROC  NEAR

	push	dx
	XCHG	AH,AL		; SAVE AL
	IN	AL,DX		; GET STATUS
	XOR	DH,DH		; START WITH ALL BITS 0
	MOV	DL,AL		; GET STATUS FROM PRINTER
	AND	AL,1		; PAPER EMPTY BIT (ACTIVE HIGH)
	MOV	CL,5
	SHL	AL,CL
	OR	DH,AL		; ADD IN PE BIT
	NOT	DL		; OTHER BITS ARE ACTIVE LOW
	MOV	AL,DL		; GET STATUS BACK
	AND	AL,4		; PRINTER SELECTED BIT
	MOV	CL,2
	SHL	AL,CL
	OR	DH,AL		; ADD IN THE SELECT BIT
	MOV	AL,DL
	AND	AL,8		; PRINTER BUSY BIT
	MOV	CL,4
	SHL	AL,CL
	OR	AL,DH		; ADD OTHER BITS TO IT
	XCHG	AH,AL		; RESTORE AL, PUT STATUS IN AH
	pop	dx
	RET
STAT_PRINT_IO   ENDP

;--INT 13 ------------------------
;DISK I/O
;
;  INPUT:
;       (AH)=0  RESET DISKETTE SYSTEM
;               HARD RESET TO 8272, SPECIFY COMMAND, RECAL REQD
;       (AH)=1  READ STATUS INTO (AL)
;               DISK_STAT FROM LAST XFR IS USED
;       (AH)=2  READ SECTORS INTO MEMORY
;       (AH)=3  WRITE SECTORS FROM MEMORY
;       (AH)=4  VERIFY SECTORS
;       (AH)=5  FORMAT A TRACK
;                 THE BUFFER POINTER (ES,BX) MUST
;                 POINT TO THE DESIRED ADDRESS FIELDS FOR THE
;                 TRACK.  EACH FIELD IS COMPOSED OF 4 BYTES, (C,H,R,N),WHERE
;                 C = TRACK NUMBER, H=HEAD NUMBER, R = SECTOR NUMBER, N=NUMBER
;                 OF BYTES/SECTOR (00=128, 01=256, 02=512, 03=1024,)
;                 THERE MUST BE ONE ENTRY FOR EVERY SECTOR ON THE TRACK.
;
;  REGISTERS FOR READ/WRITE/VERIFY/FORMAT
;       (DL) - DRIVE NUMBER (0-3)
;       (DH) - HEAD NUMBER (0-1)
;       (CH) - TRACK NUMBER (0-39)
;       (CL) - SECTOR NUMBER (1-8)
;       (AL) - NUMBER OF SECTORS (MAX=8)
;       (ES:BX) - ADDRESS OF BUFFER (NOT REQUIRED FOR VERIFY)
;
; DATA VARIABLE -- DISK_POINTER
;      DOUBLE WORD POINTER TO THE CURRENT SET OF DISKETTE PARAMETERS
;
; OUTPUT
;      AH = STATUS OF OPERATION
;      CF = 0  SUCCESSFUL OPERATION (AH=0 ON RETURN)
;      CF = 1  FAILED OPERATION (AH HAS ERROR CODE)
;      FOR READ/WRITE/VERIFY
;              DS,BX,DX,CX PRESERVED
;              AL = NUMBER OF SECTORS ACTUALLY READ
;                (AL MAY NOT BE CORRECT IF TIME OUT OCCURS)
;------------------------------------------

DISK_IO         PROC        FAR
        STI
	PUSH	DS
        PUSH    DI
        PUSH    SI
        PUSH    DX
        PUSH    CX
        PUSH    BX
	MOV	DI,DATA				; USE CORRECT DS
	MOV	DS,DI
        OR      AH,AH                       ; INIT FUNC
        JNZ     DD1
        CALL    DISK_RESET
        JMP     SHORT DISK_IO_RET
DD1:    DEC     AH                          ; GET STATUS FUNC
        JNZ     DD2
        MOV     AL,DISK_STAT
        JMP     SHORT DISK_IO_RET
DD2:    MOV     NUMSEC,AL                   ; SAVE FOR CMDS
        DEC     AH                          ; READ FUNC
        JNZ     DD3
        CALL    DISK_READ
        JMP     SHORT DISK_IO_RET
DD3:    DEC     AH                          ; WRITE FUNC
        JNZ     DD4
        CALL    DISK_WRITE
        JMP     SHORT DISK_IO_RET
DD4:    DEC     AH                          ; VERIFY FUNC
        JNZ     DD5
        CALL    DISK_VERIFY
        JMP     SHORT DISK_IO_RET
DD5:    DEC     AH                          ; FORMAT FUNC
        JNZ     DD6
        CALL    DISK_FORMAT
        JMP     SHORT DISK_IO_RET
DD6:    MOV     DISK_STAT,BAD_CMD           ; FLAG CMD ERROR

DISK_IO_RET:
        MOV     AH,DISK_STAT
        OR      AH,AH                       ; ANY ERRORS ?
        JZ      DD7                         ; NO
        STC
        JMP     SHORT DD8
DD7:    CLC
DD8:    POP     BX
        POP     CX
        POP     DX
        POP     SI
        POP     DI
	POP	DS
        RET     2                           ; USE RET 2 /NOT IRET
DISK_IO         ENDP

DISK_RESET      PROC    NEAR
        CLI
	MOV	DISK_INT_FLG,0			; CLEAR FLAG FIRST
        MOV     AL,MOTOR_STAT               ; KEEP MOTORS RUNNING
        OR      AL,8                        ; FDC RESET
        OUT     MOTOR_PORT,AL
        MOV     DISK_STAT,0                 ; CLEAR STATUS
        AND     AL,7                        ; FDC RESET BACK TO NORMAL
        OUT     MOTOR_PORT,AL
        CALL    WAIT_INT
        JC      DS1
        MOV     AH,08H                      ; SENSE INT STAT CMD
        CALL    FDC_CMD                     ; SEND TO FDC
        CALL    FDC_RSLTS                   ; GET STATUS
        JC      DS1
        MOV     RECAL_FLG,0                 ; ALL DRIVES NEED RECAL
        MOV     AH,03H                      ; SPECIFY CMD TO FDC
        CALL    FDC_CMD
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER             ; GET TABLE ADDR
        MOV     AH,ES:[SI]                  ; 1ST SPECIFY BYTE
        CALL    FDC_CMD
        MOV     AH,ES:[SI+1]                ; 2ND SPECIFY BYTE
        CALL    FDC_CMD
        POP     ES
        RET

DS1:    OR      DISK_STAT,TIME_OUT          ; FLAG TIMEOUT ERROR
        STC
        RET
DISK_RESET      ENDP

DISK_READ       PROC       NEAR
        MOV     AL,046H                     ; DMA READ
        CALL    DISK_DMA                    ; SETUP XFR
        JC      DR10                        ; BOUNDARY ERROR
        CALL    DISK_MOTOR                  ; MOTOR ON
        MOV     AH,066H                     ; READ COMMAND
        CALL    DISK_XFR                    ; DO IT
        RET

DR10:   OR      DISK_STAT,DMA_BOUNDARY      ; FLAG ERROR
        MOV     AL,0                        ; NO SECTORS XFRD
        RET
DISK_READ       ENDP

DISK_WRITE      PROC       NEAR
        MOV     AL,04AH                     ; DMA WRITE
        CALL    DISK_DMA                    ; SETUP XFR
        JC      DW10                        ; BOUNDARY ERROR
        CALL    DISK_MOTOR                  ; MOTOR ON
        MOV     AH,045H                     ; WRITE COMMAND
        CALL    DISK_XFR                    ; DO IT
        RET

DW10:   OR      DISK_STAT,DMA_BOUNDARY      ; FLAG ERROR
        MOV     AL,0                        ; NO SECTORS XFRD
        RET
DISK_WRITE      ENDP

DISK_VERIFY     PROC       NEAR
        MOV     AL,042H                     ; DMA VERIFY
        CALL    DISK_DMA                    ; SETUP XFR
        JC      DV10                        ; BOUNDARY ERROR
        CALL    DISK_MOTOR                  ; MOTOR ON
        MOV     AH,066H                     ; READ COMMAND
        CALL    DISK_XFR                    ; DO IT
        RET

DV10:   OR      DISK_STAT,DMA_BOUNDARY      ; FLAG ERROR
        MOV     AL,0                        ; NO SECTORS XFRD
        RET
DISK_VERIFY     ENDP

DISK_FORMAT     PROC       NEAR
        MOV     AL,04AH                     ; DMA WRITE
        CALL    DISK_DMA                    ; SETUP XFR
        JC      DF10                        ; BOUNDARY ERROR
        CALL    DISK_MOTOR                  ; MOTOR ON
        MOV     AH,04DH                     ; FORMAT COMMAND
        CALL    DISK_XFR                    ; DO IT
        RET

DF10:   OR      DISK_STAT,DMA_BOUNDARY      ; FLAG ERROR
        MOV     AL,0                        ; NO SECTORS XFRD
        RET
DISK_FORMAT     ENDP

DISK_DMA        PROC       NEAR
        PUSH    CX
        OUT     P8237+12,AL                 ; RESET LO/HO TOGGLE
        OUT     P8237+11,AL                 ; OUTPUT MODE BYTE
        MOV     AX,ES                       ; GET SEGMENT
        ROL     AX,1                        ; ISOLATE 4 UPPER BITS
        ROL     AX,1
        ROL     AX,1
        ROL     AX,1
        MOV     CL,AL                       ; SAVE
        AND     AL,0F0H                     ; PAGE OFFSET VALUE ONLY
        ADD     AX,BX
        JNC     DA1                         ; NO CARRY INTO PAGE VALUE
        INC     CL
DA1:    PUSH    AX                          ; SAVE ADDR
        OUT     P8237+4,AL                  ; LO ADDR
        MOV     AL,AH
        OUT     P8237+4,AL                  ; HO ADDR
        MOV     AL,CL
        AND     AL,0FH                      ; PAGE VALUE ONLY
        OUT     PAGE_PORT,AL                ; SET UP DMA PAGE REGISTER
        SUB     AX,AX
        MOV     AH,NUMSEC                   ; # OF SECTORS
        SHR     AX,1                        ; SECTORS*128
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER
        MOV     CL,ES:[SI+3]                ; GET BYTES/SECTOR CODE
        POP     ES
        SHL     AX,CL                       ; MULTIPLY BY CODE
        DEC     AX                          ; FOR DMA CHIP ARITHMETIC
        PUSH    AX                          ; STORE COUNT
        OUT     P8237+5,AL                  ; OUTPUT LO COUNT
        MOV     AL,AH
        OUT     P8237+5,AL                  ; OUTPUT HO COUNT
        POP     AX                          ; GET COUNT
        POP     CX                          ; GET ADDR
        ADD     AX,CX                       ; SET CF IF OVERFLOW
        MOV     AL,2
        OUT     P8237+10,AL                 ; INIT CHANNEL
        POP     CX
        RET
DISK_DMA        ENDP

DISK_MOTOR      PROC       NEAR
        PUSH    CX
;
;	FIRST RESET MOTOR_ON COUNTER SO THE REAL TIME INTERRUPT DOESN'T
;	TURN THE MOTORS OFF
;
	PUSH	ES
	MOV	SI,ABS0
	MOV	ES,SI
	LES	SI,DISK_POINTER
	MOV	CL,ES:[SI+2]
	POP	ES
	MOV	MOTOR_CNT,CL
;
;	NOW TURN ON THE MOTORS IF NOT ALREADY ON
;
        MOV     AL,DL                       ; GET DRIVE #
        INC     AL
        INC     AL
        CMP     AL,3                        ; DRIVE B ?
        JG      DM1                         ; DRIVE C/D
        JL      DM2                         ; DRIVE A
        MOV     AL,3                        ; LOAD MASK
        JMP     SHORT DM3
DM1:    MOV     AL,4
        JMP     SHORT DM3
DM2:    MOV     AL,3                        ; LOAD MASK
DM3:    TEST    AL,MOTOR_STAT               ; MOTOR ON ?
        JNZ     DM6                         ; YES
        MOV     AH,MOTOR_STAT               ; FLAG ON NOW
        OR      AL,AH
	MOV	MOTOR_STAT,AL
        OUT     MOTOR_PORT,AL
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER
        MOV     AH,ES:[SI+10]               ; GET WAIT TIME
        POP     ES
DM4:    OR      AH,AH                       ; NO MORE WAIT ?
        JZ      DM6
        XOR     CX,CX
DM5:    LOOP    DM5
        DEC     AH                          ; COUNT EACH 1/8 SEC
        JMP     DM4
DM6:    POP     CX
        RET
DISK_MOTOR      ENDP

DISK_XFR        PROC      NEAR
        PUSH    AX                          ; SAVE COMMAND
        PUSH    CX
        MOV     AL,1                        ; CREATE DRIVE MASK
        MOV     CL,DL                       ; GET DRIVE #
        ROL     AL,CL
        POP     CX
        TEST    AL,RECAL_FLG                ; RECALIBRATE ?
        JNZ     DX1B                        ; NOT NEEDED
        OR      RECAL_FLG,AL                ; SET FLAG
        MOV     AH,07H                      ; RECALIBRATE
        CALL    FDC_CMD
        MOV     AH,DL
        CALL    FDC_CMD
        CALL    WAIT_INT
        JNC     DX1A
DX1:    OR      DISK_STAT,TIME_OUT          ; FLAG ERROR
        POP     AX                          ; CLEAN UP STACK
        RET
DX1A:   MOV     AH,08H                      ; SENSE INT STATUS
        CALL    FDC_CMD
        CALL    FDC_RSLTS
        JC      DX1                         ; ERROR
DX1B:   MOV     AH,0FH                      ; SEEK COMMAND
        CALL    FDC_CMD
        MOV     AH,DL                       ; DRIVE #
        CALL    FDC_CMD
        MOV     AH,CH                       ; TRACK #
        CALL    FDC_CMD
        CALL    WAIT_INT
        JC      DX1                         ; TIMEOUT ERROR
        MOV     AH,08H                      ; SENSE INT STAT
        CALL    FDC_CMD
        CALL    FDC_RSLTS
        JC      DX1                         ; ERROR
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER
        MOV     AH,ES:[SI+9]                ; GET HEAD SETTLE TIME
        POP     ES
        PUSH    CX
DX2:    MOV     CX,550                      ; VALUE FOR 1 MS LOOP
        OR      AH,AH
        JZ      DX3A                        ; DONE
DX3:    LOOP    DX3
        DEC     AH                          ; COUNT MS
        JMP     SHORT DX2
DX3A:   POP     CX
        POP     AX                          ; GET COMMAND
        CMP     AH,04DH                     ; FORMAT ?
        JE      DX4
        CALL    FDC_CMD                     ; SEND CMD BYTE
        MOV     AH,DH                       ; GET HEAD #
        SAL     AH,1
        SAL     AH,1
        AND     AH,4                        ; ISOLATE HEAD BIT
        AND     DL,03H                      ; ISOLATE DRIVE #
        OR      AH,DL                       ; MAKE HEAD/DRIVE #
        CALL    FDC_CMD
        MOV     AH,CH                       ; GET CYLINDER #
        CALL    FDC_CMD
        MOV     AH,DH                       ; GET HEAD #
        CALL    FDC_CMD
        MOV     AH,CL                       ; GET SECTOR #
        CALL    FDC_CMD
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER
        MOV     AH,ES:[SI+3]                ; GET BYTES/SECTOR
        CALL    FDC_CMD
        MOV     AH,ES:[SI+4]                ; GET # OF SECTORS
        CALL    FDC_CMD
        MOV     AH,ES:[SI+5]                ; GET GAP LEN
        CALL    FDC_CMD
        MOV     AH,ES:[SI+6]                ; GET DTL
        CALL    FDC_CMD
        POP     ES
        JMP     DX5
DX4:    CALL    FDC_CMD                     ; SEND COMMAND BYTE
        MOV     AH,DH                       ; GET HEAD #
        SAL     AH,1
        SAL     AH,1
        AND     AH,4                        ; ISOLATE HEAD BIT
        AND     DL,03H                      ; ISOLATE DRIVE #
        OR      AH,DL                       ; MAKE HEAD/DRIVE #
        CALL    FDC_CMD
        PUSH    ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES     SI,DISK_POINTER
        MOV     AH,ES:[SI+3]                ; GET BYTES/SECTOR
        CALL    FDC_CMD
        MOV     AH,ES:[SI+4]                ; GET # OF SECTORS
        CALL    FDC_CMD
        MOV     AH,ES:[SI+7]                ; GET FORMAT GAP LEN
        CALL    FDC_CMD
        MOV     AH,ES:[SI+8]                ; GET FILLER BYTE
        CALL    FDC_CMD
        POP     ES
DX5:    CALL    WAIT_INT
        JNC     DX6                         ; NO ERROR
        CALL    FDC_RSLTS                   ; FLUSH CHIP
        RET
DX6:    CALL    FDC_RSLTS                   ; GET STATUS
        JNC     DX7                         ; EVERYTHING IS OKAY
        RET
DX7:    MOV     AL,DISK_STAT_ARRAY          ; GET STATUS
        AND     AL,0C0H                     ; NORMAL ?
        JNZ     DX9                         ; ERROR, GO GET SPECIFIC

DX7A:   MOV    AL,DISK_STAT_ARRAY+3         ; GET FINAL TRACK
        CMP    AL,CH                        ; SAME AS FIRST ?
        MOV    AL,DISK_STAT_ARRAY+5         ; GET FINAL SECTOR
        JZ     DX8                          ; DON'T ADJUST
        PUSH   ES
	MOV	SI,ABS0
	MOV	ES,SI
        LES    SI,DISK_POINTER
        MOV    AL,ES:[SI+4]                 ; GET # OF SECTORS
        POP    ES
        INC    AL                           ; USE EOT+1 FOR CALCULATION
DX8:    SUB    AL,CL                        ; SUBTRACT START FROM END
        RET

DX9:    MOV    AL,DISK_STAT_ARRAY+1         ; GET STATUS 1
        TEST   AL,080H                      ; EOT ?
        MOV    AH,RECORD_NOT_FND
        JNZ    DX9A
        TEST   AL,020H                      ; CRC BAD ?
        MOV    AH,BAD_CRC
        JNZ    DX9A
        TEST   AL,010H                      ; DMA BAD ?
        MOV    AH,BAD_DMA
        JNZ    DX9A
        TEST   AL,04H                       ; RECORD NOT FOUND ?
        MOV    AH,RECORD_NOT_FND
        JNZ    DX9A
        TEST   AL,02H                       ; WRITE PROTECT ?
        MOV    AH,WRITE_PROTECT
        JNZ    DX9A
        TEST   AL,01H                       ; MISSING ADDR MARK ?
        MOV    AH,BAD_ADDR_MARK
        JNZ    DX9A
        MOV    AH,BAD_8272                  ; CATCH-ALL ERROR
DX9A:   OR     DISK_STAT,AH                 ; SET ERROR FLAG
        JMP    DX7A                         ; GO EXIT

DISK_XFR        ENDP

FDC_CMD         PROC       NEAR
        PUSH    CX
        XOR     CX,CX                       ; CLR FOR TIMEOUT
DN1:    IN      AL,P8272_STAT               ; GET STATUS
        AND     AL,0C0H
        XOR     AL,080H
        JZ      DN2                         ; OKAY
        LOOP    DN1
        OR      DISK_STAT,TIME_OUT          ; FLAG TIMEOUT
        STC
        POP     CX
        RET
DN2:    MOV     AL,AH
        OUT     P8272_DATA,AL               ; OUTPUT COMMAND BYTE
        POP     CX
        RET
FDC_CMD         ENDP

FDC_RSLTS       PROC        NEAR
        PUSH    CX
        MOV     SI,OFFSET DISK_STAT_ARRAY
        XOR     CX,CX
DR1:    IN      AL,P8272_STAT               ; GET STATUS
        AND     AL,0C0H
        XOR     AL,0C0H
        JZ      DR2                         ; OKAY
        LOOP    DR1
        OR      DISK_STAT,TIME_OUT          ; FLAG TIMEOUT
        STC
        POP     CX
        RET
DR2:    IN      AL,P8272_DATA               ; GET STATUS FROM CHIP
        MOV     [SI],AL                     ; STORE IN ARRAY
        INC     SI
        MOV     CX,010H
DR3:    LOOP    DR3                         ; KILL TIME
        IN      AL,P8272_STAT               ; GET STATUS
        AND     AL,010H                     ; FDC STILL BUSY ?
        JNZ     DR1                         ; YES        
        POP     CX
        RET
FDC_RSLTS       ENDP

WAIT_INT        PROC       NEAR
        STI
        PUSH    CX
	PUSH	AX
        XOR     CX,CX                       ; FOR TIMEOUT
	MOV	AX,CX
DW1:    CMP     DISK_INT_FLG,0FFH
        JE      DW2
        LOOP    DW1
	CMP	AL,0
	JNE	DW3
	INC	AL
	XOR	CX,CX
	JMP	SHORT DW1
DW3:    OR      DISK_STAT,TIME_OUT          ; FLAG ERROR
        STC
	JMP	SHORT DW4
DW2:    MOV     DISK_INT_FLG,0              ; CLEAR FLAG FOR NEXT TIME
DW4:	POP	AX
        POP     CX
        RET
WAIT_INT        ENDP

DISK_INT        PROC       FAR
        STI
        PUSH    AX
        PUSH    DX
        PUSH    DS
        MOV     AX,DATA
        MOV     DS,AX
        MOV     DISK_INT_FLG,0FFH           ; FLAG THAT INT OCCURRED
        MOV     DX,P8259_M0
        MOV     AL,061H
        OUT     DX,AL                       ; RESET MASTER 8259
        MOV     DX,P8259_S0
        MOV     AL,065H
        OUT     DX,AL                       ; RESET SLAVE 8259
        POP     DS
        POP     DX
        POP     AX
        IRET
DISK_INT        ENDP

;----------------------------
; INTERRUPT VECTOR TABLE
;----------------------------
	org	01f13h		; locate same as in IBM PC
;
IBM_VEC_TBL  EQU    $
;
    DW    OFFSET VID_INT_HDLR   ; IBM COMPATIBLE VIDEO I/O (INT 10H)
    DW    CODESG
    DW    OFFSET IBM_EQUIP      ; IBM COMPATIBLE EQUIP CHK (INT 11)
    DW    CODESG
    DW    OFFSET IBM_MEMORY     ; IBM COMPATIBLE MEM CHK   (INT 12)
    DW    CODESG
    DW    OFFSET DISK_IO        ; IBM COMPATIBLE DISK I/O  (INT 13)
    DW    CODESG
    DW    OFFSET SER_IO         ; IBM COMPATIBLE RS232 I/O (INT 14)
    DW    CODESG
    DW    OFFSET IBM_CASSETTE   ; IBM COMPAT CASSETTE I/O  (INT 15)
    DW    CODESG
    DW    OFFSET KB_IO          ; IBM COMPAT KEYBOARD I/O  (INT 16H)
    DW    CODESG
    DW    OFFSET PRINT_IO       ; IBM COMPAT PRINTER I/O   (INT 17)
    DW    CODESG
    DW    OFFSET IBM_BASIC      ; IBM ROM BASIC ENTRY      (INT 18)
    DW    CODESG
    DW    OFFSET BOOT_STRAP     ; IBM COMPAT BOOTSTRAP     (INT 19)
    DW    CODESG
    DW    OFFSET TIME_DAY       ; TIME OF DAY              (INT 1A)
    DW    CODESG
    DW    OFFSET IBM_KB_BREAK   ; IBM COMPAT KB BREAK ADDR (INT 1B)
    DW    CODESG
    DW    OFFSET IBM_TIME_BREAK ; IBM COMPAT TIMER BRK ADR (INT 1C)
    DW    CODESG
    DW    OFFSET VID_PARM_TABLE ; IBM COMPAT VIDEO PARAMS  (INT 1D)
    DW    CODESG
    DW    OFFSET DISK_BASE      ; IBM COMPAT DISK TABLE    (INT 1E)
    DW    CODESG
    DW    OFFSET IBM_VID_EXT    ; IBM COMPAT VIDEO EXTENS  (INT 1F)
    DW    CODESG
END_IBM_TBL    EQU    $
IBM_TBL_SIZ    EQU    (END_IBM_TBL-IBM_VEC_TBL)/2

;
; 20H-3FH RESERVED FOR MS DOS
;
;
; BEGIN MPX HARD INTERRUPTS
;
MPX_VEC_TBL    EQU    $
;
;*	Master 8259a interrupt controller vectors.
;
    DW    OFFSET REAL_TIME      ; REAL TIME INTERRUPT - INT 40H
    DW    CODESG
    dw    offset TRAP_INT	; slave 8259a interrupt pin
    dw    CODESG
    DW    OFFSET KB_INT_HDLR	; INT 042H - MPX-16 KBD ADAPTER
    DW    CODESG
    dw    offset TRAP_INT	; i/o chnl irqst3 
    dw    CODESG
    dw    offset TRAP_INT	; i/o chnl irqst4
    dw    CODESG
    dw    offset TRAP_INT	; i/o chnl irqst5
    dw    CODESG
    dw    offset TRAP_INT	; i/o chnl irqst6
    dw    CODESG
    dw    offset TRAP_INT	; i/o chnl irqst7
    dw    CODESG
;
;*	Slave 8259a interrupt controller vectors
;
    DW    OFFSET ser_con_in	; CONSOLE IN            INT 48
    DW    CODESG
    DW    OFFSET TRAP_INT       ; AUX IN                INT 49
    DW    CODESG
    DW    OFFSET ser_con_out    ; CONSOLE OUT           INT 4A
    DW    CODESG
    DW    OFFSET TRAP_INT       ; AUX OUT               INT 4B
    DW    CODESG
    DW    OFFSET TRAP_INT       ; PRINTER OUT           INT 4C
    DW    CODESG
    DW    OFFSET DISK_INT       ; P8272 INTERRUPT       INT 4D
    DW    CODESG
    dw    offset TRAP_INT	; INT 4EH
    dw    CODESG
    dw    offset TRAP_INT	; INT 4FH
    dw    CODESG
;
END_MPX_TBL    EQU    $
MPX_TBL_SIZ    EQU    (END_MPX_TBL-MPX_VEC_TBL)/2

;------------------------------------------------
; DISK_BASE
;------------------------------------------------

DISK_BASE    LABEL    BYTE
          DB    11001111B    ; 1ST SPECIFY BYTE
          DB    2            ; 2ND SPECIFY BYTE
          DB    025H         ; WAIT UNTIL MOTOR OFF
          DB    2            ; 512 BYTES/SECTOR
          DB    8            ; EOT
          DB    02AH         ; GAP
          DB    0FFH         ; DTL
          DB    050H         ; GAP FOR FORMAT
          DB    0F6H         ; FILL FOR FORMAT
          DB    25           ; HEAD SETTLE TIME (MILLISECONDS)
          DB    4            ; MOTOR START TIME (1/8 SECONDS)
;
BAUD_RATE	label	word
;
public	BAUD_RATE
;
	dw	698		; 110
	dw	512		; 150
	dw	256		; 300
	dw	128		; 600
	dw	64		; 1200
	dw	32		; 2400
	dw	16		; 4800
	dw	8		; 9600
;
EXP_BAUDRATES	label	word
;
	dw	1047		; 110
	dw	768		; 150
	dw	384		; 300
	dw	192		; 600
	dw	96		; 1200
	dw	48		; 2400
	dw	24		; 4800
	dw	12		; 9600
;
START	ENDP
;
CSEG	ENDS
            END START
