
title	Graphics adapter board driver routines
page	60,132
;*
;**********************************************************************
;*
;*      module name - vidvxx.a86        xx = revision number
;*
;*      target machine - the Micromint MPX-16
;*
;*      language - Intel 8088 assembly language 
;*                 assembled using the Microsoft MACRO assembler
;*
;*      purpose - support of an IBM Personal Computer compatible
;*                monochrome or color adapter module
;*
;*      module origination date - 4/25/83
;*
;*      date of this revision:	11/03/83
;*
;**********************************************************************
;*
;*      video graphics module equates
;*
monochrome              equ     030h    ; mask for equipment_flag
monochrome_buffer_size  equ     2048    ; 4k bytes at 0b000:0000
color_buffer_size       equ     8192    ; 16k bytes at 0b800:0000
hi_res_mode             equ     6       ; 640x200 graphics mode
monochrome_mode         equ     7       ; monochrome card is in use
bw_base                 equ     03b4h   ; monochrome adapter card
color_base              equ     03d4h   ; color adapter card
bw_vram_base            equ     0B000h  ; video ram base
color_vram_base         equ     0B800h  ; video ram base
blank_char              equ     0720h   ; ascii space with attrib 7
cursor_type_reg         equ     10      ; 6845 cursor type register
crt_start_address       equ     12      ; 6845 start addr register
cursor_posn_reg         equ     14      ; 6845 cursor posn register
display_enable_bit      equ     1       ; bit 1 of the status port
cr                      equ     0Dh     ; carriage return
lf                      equ     0Ah     ; line feed
backspace               equ     8       ; backspace ascii code
bell                    equ     7       ; ascii bel character
space                   equ     020h    ; ascii space character
write_a_pixel		equ	12	; Function number 12
;*
;*	external and public symbol definitions
;*
data	segment	at	040h
;
extrn	video_mode:byte,number_of_display_columns:word
extrn	display_ram_size:word,display_start_address:word
extrn	cursor_position_table:word,cursor_type:word
extrn	active_display_page:byte,video_port_base:word
extrn	current_control_setting:byte,current_color_pallette:byte
extrn	temp_char:byte,equip_flag:word,display_lines:byte
;
data	ends
;
abs0	segment at	0
extrn	vid_parm_ptr:dword,gr_char_ext:dword
abs0	ends
;
extrn	graphics_char_table:byte,baud_rate:word
public	vid_int_hdlr,ser_con_out
extrn	play_tone:far
;
cseg	segment	common
assume	cs:cseg,ds:data
;
	org	0580h		; start module at FE00:0580
;*
;**********************************************************************
;*
;*      Vid_int_hdlr:
;*
;*      This module processes software interrupt calls through the
;*      int 010h interrupt vector.
;*
;*      Sixteen seperate function calls are supported.  The function
;*      to be processed is determined by the value passed in the
;*      AH register (only 0 to 15 are valid entries).  Depending on the
;*      requirements of each particular function, one or more of the
;*      general registers may be used to pass data to and from the
;*      invoking procedure.  Registers used by each function are listed
;*      in the prolog for the function below.
;*
;*      The supported functions are compatible with the IBM Personal
;*      computer:
;*
;*      AH/fn #                 description
;*
;*        0             sets a new operating mode
;*        1             sets the cursor type/size
;*        2             sets the cursor position
;*        3             read the current cursor position
;*        4             read the light pen position
;*        5             selects the active display page
;*        6             scrolls the active display page up
;*        7             scrolls the active display page down
;*        8             read character and attribute at cursor position
;*        9             write char. and attribute at cursor position
;*       10             write character only at cursor position
;*       11             sets a new color pallette
;*       12             write a new pixel
;*       13             read a pixel
;*       14             emulates tty output to the screen
;*       15             returns the current video state
;*
;**********************************************************************
;
vid_int_hdlr	proc	far
;
        sti             ; allow further system interrupts to occur
        push    ds      ; save all registers
        push    es
        push    si
        push    di
        push    bx
        push    cx
        push    dx
        push    ax
;
	mov	ax,data			; set up for bios ram area
        mov     ds,ax
        mov     ax,equip_flag	; b/w or color card ?
        and     ax,030h		; check the monitor mode bits
        jne     set_up_vram
;
	pop	ax		; restore function code
	cmp	ah,0		; initialize ?
	jne	ck_out_fns
	call	serial_out_init
	jmp	video_exit
ck_out_fns:
	cmp	ah,9		; write char and attribute ?
	je	output_serial_char
	cmp	ah,10		; write char only ?
	je	output_serial_char
	cmp	ah,14		; emulate tty mode ?
	jne	serial_term_exit
output_serial_char:
	call	serial_con_output
serial_term_exit:
	jmp	video_exit
;
set_up_vram:
	cmp	ax,monochrome
	jne	color_card_attached
;
        mov     ax,bw_vram_base         ; b/w video ram address
        jmp    short compute_function_address
;
color_card_attached:
;
        mov     ax,color_vram_base      ; color video ram address
;
compute_function_address:
;
        mov     es,ax           ; first set up video ram segment
        pop     ax              ; restore function parameters
        push    ax              ; and save them again
        mov     al,ah           ; put function code in low byte
        xor     ah,ah
        shl     ax,1            ; *2 for word offsets
        mov     si,ax           ; use si register as a table offset
        pop     ax              ; restore for function calls
	jmp	cs:video_jump_table[si]	; execute indicated function
;
;**********************************************************************
;*
;*      Function 0 - set the video mode
;*
;*      On entry:
;*
;*              The code in AL determines the mode as:
;*
;*              0 - 40x25 b/w
;*              1 - 40x25 color
;*              2 - 80x25 b/w
;*              3 - 80x25 color
;*              4 - 320x200 pixel color graphics
;*              5 - 320x200 b/w graphics
;*              6 - 640x200 b/w graphics
;*              7 - 80x25 monochrome adapter - used internally only
;*
;*      On exit:
;*
;*              Nothing is returned, AX register is destroyed.
;*
;**********************************************************************
;
set_video_mode:
;
        cld             ; go forward for string instructions
        mov     bl,al   ; save mode in BL
        xor     bh,bh   ; clear upper byte for later use
        mov     ax,es   ; test for monochrome or color adapter
        cmp     ax,bw_vram_base ; address was previously set up
        jne     select_color_port
;*
;*      Set up for the 80x25 monochrome adapter board
;*
        mov     dx,bw_base      ; base i/o port address
        mov     bl,monochrome_mode      ; b/w card mode value
        mov     al,1            ; port initialization value
        jmp    short save_mode_and_base
;*
;*      Set up for the color adapter card
;*
select_color_port:
;
        mov     dx,color_base   ; base i/o port address
        xor     al,al           ; color card mode value
;
save_mode_and_base:
;
        mov     video_mode,bl   ; save the current mode value
        mov     video_port_base,dx      ; save the video i/o base addr
;*
;*      Disable and initialize the video adapter control register.
;*
        add     dx,4            ; base+4 = control register
        out     dx,al
;*
;*      Initialize system variables to 0, then clear the 
;*      display ram by filling with blanks.
;*
        xor     ax,ax
        mov     active_display_page,al
        mov     display_start_address,ax
        mov     di,ax           ; use di for offset in display ram
;
        cmp     bl,monochrome_mode      ; monochrome adapter selected ?
        jne     clear_color_ram
;
        mov     cx,monochrome_buffer_size       ; monochrome card
        mov     ax,blank_char                   ; ascii space
        jmp    short fill_video_ram
;
clear_color_ram:
;
        mov     cx,color_buffer_size            ; color card here
        cmp     bl,4                            ; graphics mode ?
        jae     fill_video_ram                  ; fill with 0's if so
;
        mov     ax,blank_char                   ; ascii space
;
fill_video_ram:
;
        rep     stosw
;*
;*      Initialize the 6845 CRT controller from the table of video
;*      parameters pointed to by the interrupt vector at 01dh.
;*      First set up the data segment for absolute 0.
;*
initialize_6845:
;
        push    ds
	xor	ax,ax
	mov	ds,ax			; abs0 segment
assume	ds:abs0
        lds     si,vid_parm_ptr
        mov     al,bl                   ; get the current mode
        shr     al,1                    ; 2 modes per table entry except
        cmp     bl,6                    ; for special case for mode 6
        jne     compute_video_parameters
;
        mov     al,2                    ; for graphic modes (4,5,6)
;
compute_video_parameters:
;
assume	ds:cseg
        mov     cx,video_parm_length
        mul     cl                      ; point to correct table
        add     si,ax
;
        xor     ah,ah                   ; use ah for 6845 register #
        sub     dx,4                    ; set port addr back to base
;
init_6845_loop:
;
        mov     al,ah                   ; get next register number
        out     dx,al                   ; write it to 6845
        inc     dx                      ; point now to data register
        mov     al,[si]                 ; get next parameter
        out     dx,al
        inc     ah                      ; next 6845 index register
        dec     dx                      ; port back to index register
        inc     si                      ; next parameter in table
        loop    init_6845_loop
;
        pop     ds                      ; restore segment
assume	ds:data
;*
;*      Now output the video control parameter for the selected mode.
;*
        add     dx,4                    ; port addr set to control reg
	mov	al,cs:video_control_table[bx]	; bx contains the mode
        out     dx,al
        mov     current_control_setting,al      ; save the value
;*
;*      Determine number of columns in display and save it.
;*
	mov	al,cs:columns_table[bx]
        xor     ah,ah
        mov     number_of_display_columns,ax
;*
;*      Initialize the cursor positions to location 0 in display ram.
;*
        mov     cursor_type,0607h       ; this was already set above
        mov     cx,8                    ; up to 8 display pages
        mov     di,offset cursor_position_table
        mov     ax,ds                   ; set segment for video ram
        mov     es,ax
        xor     ax,ax                   ; set cursor posn's to 0
        rep     stosw
;*
;*      Determine display memory size and save the value.
;*
        and     bx,0eh          ; convert for every other entry
	mov	cx,cs:display_length_table[bx]
        mov     display_ram_size,cx
;*
;*      Initialize the overscan/color pallette port.
;*
        cmp     video_mode,hi_res_mode
        jne     mode_not_640x200
;
        mov     al,03fh         ; control value for 600x200 mode
        jmp    short write_new_pallette
;
mode_not_640x200:
;
        mov     al,030h         ; control value for other modes
;
write_new_pallette:
;
        inc     dx              ; bump to color pallette port
        out     dx,al
        mov     current_color_pallette,al       ; save it
;
        jmp     video_exit      ; done with this function call
;
;**********************************************************************
;*
;*      Function 1 - sets the cursor type/size
;*
;*      On entry:
;*
;*              CX contains the cursor size, starting scan line is in
;*              CH, ending scan line is in CL.
;*
;*      On exit:
;*
;*              Nothing is returned, AX and DX are destroyed.
;*
;**********************************************************************
;
set_cursor_type:
;
;*
;*      First save the cursor type, then call a routine to output the
;*      data in the CX register to the 6845 CRT controller.  The cursor
;*      register addresses for the starting and ending scan lines  in
;*      the 6845 are at 10 and 11, respectively.
;*
        mov     ah,cursor_type_reg      ; 6845 index register value
        mov     cursor_type,cx
        call    output_cx_to_6845
;
        jmp     video_exit
;
;**********************************************************************
;*
;*      Function 2 - sets a new cursor position in the active display
;*                   page specified in BH.
;*
;*      On entry:
;*
;*              DX contains the new row (DH) and column (DL),
;*              BH contains the display page number.
;*
;*      On exit:
;*
;*              Cursor position is moved if page number is the same as
;*              the current active page number.
;*
;**********************************************************************
;
set_cursor_position:
;*
        call    update_cursor_position  ; routine also used by fn# 14
        jmp     video_exit
;
;**********************************************************************
;*
;*      Function 3 - reads the cursor position of the page specified
;*                   in the BH register.
;*
;*      On entry:
;*
;*              BH contains the page number - 0 to 7 for character
;*              modes, 0 only for graphics modes
;*
;*      On exit:
;*
;*              CX contains the current cursor type/size
;*              DX contains the position of the cursor - row in DH,
;*              column in DL
;*
;**********************************************************************
;
read_cursor_position:
;*
;*      First remove the old values of CX and DX from the stack.
;*      then push the cursor type and positions onto to the stack.
;*      The parameters will then be returned when the CX and DX
;*      registers are popped from the stack in the int 010h exit
;*      point.
;*
        pop     dx
        pop     cx
;
        push    cursor_type     ; first put the cursor type on the stack
;*
;*      Now push the cursor position for the page onto the stack.
;*
        mov     bl,bh           ; put page in low byte for table offset
        xor     bh,bh
        shl     bx,1            ; *2 for word offsets
        push    cursor_position_table[bx]
;
        jmp     video_exit
;
;*
;**********************************************************************
;*
;*      Function 4 - reads the position of the light pen register.
;*
;**********************************************************************
;*
read_light_pen_position:
;*****	implement later *********************************
;
	xor	ah,ah	; set for not available for now
        jmp     video_exit
	db	175 dup(0FFH)	; reserve space for light pen routine
;
;**********************************************************************
;*
;*      Function 5 - selects a new display page in character modes
;*
;*      On entry:
;*
;*              AL contains the new page number, 0 to 7 for 40x25
;*              modes, 0 to 3 for 80x25 modes.
;*
;*      On output:
;*
;*              The 6845 CRT controller will be displaying the page
;*              which was selected.
;*
;**********************************************************************
;
select_display_page:
;*
;*      First, save the new page number.  Then compute the new starting
;*      address of the page in video ram and output it to the 6845 CRT
;*      controller.  Finally, update the cursor to the position for the
;*      new page.
;*
        mov     active_display_page,al  ; save the new page number
;*
;*      Compute the starting address = page# x #bytes per screen
;*
        xor     ah,ah
        mov     bx,ax           ; save the page number
        mul     word ptr display_ram_size
        mov     display_start_address,ax        ; save start address
;*
;*      Now output the address to the 6845 CRT controller.
;*
        shr     ax,1            ; 2 bytes per character - divide by 2
        mov     cx,ax           ; put in cx for subroutine call
        mov     ah,crt_start_address    ; 6845 index register number
        call    output_cx_to_6845
;*
;*      Get the cursor position for this page and output it.
;*
        shl     bx,1            ; *2 for word offsets in table
        mov     dx,cursor_position_table[bx]    ; row,col in dx
        call    move_the_cursor
;
        jmp     video_exit
;*
;**********************************************************************
;*
;*      Function 6 - scrolls a block of text up on the screen, or clears
;*                   the entire block if the row count is zero.
;*
;*      On entry:
;*
;*              AL contains the number of rows to scroll, (or blank the
;*              entire window if set to 0),
;*              CX contains the upper left corner, row in CH, col in CL,
;*              DX contains the lower right corner, row in DH, col in DL
;*              BH contains the fill attribute for blanked lines.
;*
;*      On exit:
;*
;*              The display will have been scrolled up by the specified
;*              number of lines.
;*
;**********************************************************************
;*
scroll_page_up:
;*
        cld             ; clear direction flag to forward for up
        xor     bl,bl   ; clear flag used by common scroll routine
        call    scroll_handler  ; go do the scroll
;
        jmp     video_exit
;*
;**********************************************************************
;*
;*      Function 7 - scrolls the active display page down by a specified
;*                   number of lines, or clears the entire area of text
;*                   if the line count is zero.
;*
;*      On entry:
;*
;*              AL contains the number of lines to scroll,
;*              CX contains the upper left corner, row in CH, col in CL
;*              DX contains the lower right corner, row in DH, col in DL
;*              BH contains the fill attribute for blanked lines.
;*
;*      On exit:
;*
;*              Nothing is returned, the display will have been scrolled
;*              down by the specified number of lines.
;*
;**********************************************************************
;*
scroll_page_down:
;*
        std             ; set direction flag for down
        mov     bl,-1   ; set flag used by common scroll routine
        call    scroll_handler  ; go do the scroll
;
        jmp     video_exit
;
;**********************************************************************
;*
;*      Function 8 - returns the character and attribute at the cursor
;*      position of the specified display page.
;*
;*      On entry:
;*
;*              BH contains the display page if not in graphics mode,
;*
;*      On exit:
;*
;*              AL contains the character code,
;*              AH contains the attribute for the character
;*
;**********************************************************************
;
read_char_and_attribute:
;*
;*      First determine whether to read a character in alphanumeric
;*      mode, or in a graphics mode, since each has to be handled in a
;*      different manner.
;*
        cmp     video_mode,4            ; graphics modes are 4,5,6
        jb      find_alpha_read_posn
;
        cmp     video_mode,monochrome_mode      ; monochrome card ?
        je      find_alpha_read_posn
;
        jmp     short graphics_mode_read
;
find_alpha_read_posn:
;*
;*      Get the video memory address at the cursor position and put it
;*      in the SI register for the lodsw instruction.
;*
        call    compute_cursor_address
        mov     si,di                   ; result was returned in DI
;*
;*      Now look at the display enable bit of the status port to do
;*      the read during a horizontal retrace to avoid display
;*      interference.
;*
        mov     dx,video_port_base      ; 03b4h or 03d4h
        add     dx,6                    ; video card status port
        mov     ax,es                   ; lodsw uses the DS segment
        mov     ds,ax
;
wait_for_dispen_low:
;
        in      al,dx                   ; wait for low level
        test    al,display_enable_bit   ; bit 1
        jnz     wait_for_dispen_low
        cli             ; don't want to be interrupted here
;
wait_for_dispen_high:
;
        in      al,dx                   ; now wait until it is high
        test    al,display_enable_bit
        jz      wait_for_dispen_high
;
        lodsw           ; now quickly read the location
        sti             ; now that char is read, turn interrupts back on
;
        jmp     video_exit
;
graphics_mode_read:
;
;*      First get the video ram location at the current cursor position.
;*      The byte position of the first even scan row will be returned in
;*      the SI register.
;
        call    compute_graphics_cursor_posn
        cmp     video_mode,hi_res_mode
        je      read_a_hi_res_char
;
;*      At this point a medium resolution graphics character will be read
;*      into the 8-byte temporary array, temp_char.  Each character uses
;*      16 bits in four even and four odd scan rows.  The 16 bits must
;*      first be compressed into 8-bit values for comparison with the
;*      characters in the ROM and user character tables.
;
        mov     di,si           ; save the pointer
        xor     bx,bx           ; pointer into temp_char array
        mov     dh,bh           ; byte counter
;
form_med_res_byte:
;
        xor     dl,dl           ; bit counter
        mov     ch,es:[si]      ; get the first 4 pixels
        mov     cl,es:1[si]     ; get the second 4 pixels
;
check_if_background:
;
        shl     al,1            ; shift for processing next bit
        test    cx,0c000h       ; background color if both are 0
        jz      set_for_background
;
        or      al,1            ; set the bit since not background
        jmp    short check_if_8_bits_done
;
set_for_background:
;
        and     al,0feh         ; clear the bit for background
;
check_if_8_bits_done:
;
        shl     cx,1            ; shift to next pixel
        shl     cx,1            ; 2 bits per pixel
        inc     dl              ; bump the bit counter
        cmp     dl,8            ; finished all eight?
        jb      check_if_background
;
        mov     temp_char[bx],al        ; save the 8-bit value
        add     si,080          ; go to next row of character
        inc     dh              ; bump byte counter
        add     bx,2            ; do all even scans, then all odd
        cmp     dh,4            ; finished with even scans?
        jne     check_if_done
;
        mov     bx,1            ; now do the odd scan bytes
        mov     si,di           ; set up initial pointer in video ram
        add     si,02000h
;
check_if_done:
;
        cmp     dh,8            ; 8 bytes total to do
        jb      form_med_res_byte
;
        jmp    short check_if_rom_char       ; now get the character code
;
read_a_hi_res_char:
;
        mov     di,offset temp_char     ; temporary 8-byte buffer
        xor     ch,ch                   ; byte counter
;
hi_res_read_loop:
;
        test    ch,1                    ; even=even scan, odd=odd scan
        jnz     read_hi_res_odd_scan
;
        mov     al,es:[si]
        mov     [di],al
        jmp    short bump_read_ptrs
;
read_hi_res_odd_scan:
;
        mov     al,es:2000H[si]
        mov     [di],al
        add     si,80                   ; increment to next scan row
;
bump_read_ptrs:
;
        inc     di                      ; next temp location
        inc     ch                      ; next byte
        cmp     ch,8                    ; 8 total per character
        jb      hi_res_read_loop
;*
;*      Now that an 8-byte graphics character has been read from the video
;*      ram, it can be compared first to the table of characters stored
;*      as graphics_char_table in ROM (codes 0 through 127), and then to
;*      the user character table pointed to by interrupt vector at 07ch
;*      (codes 128 through 255).
;*
check_if_rom_char:
;
        mov     ax,cs           ; set up pointers for CMPSB instruction
        mov     es,ax
        mov     si,offset temp_char
        mov     di,offset graphics_char_table
        xor     al,al           ; use al as a counter to return char code
;
find_char_loop:
;
        mov     cx,8            ; 8 bytes per character code
        push    si              ; save pointer value
        push    di              ; save this one too
        repe    cmpsb           ; check for match of all 8 bytes
        jz      return_graphics_char_code
;
        inc     al              ; bump to next char code
        jz      return_graphics_char_code       ; not found if 0 here
;
        pop     di              ; bump pointer to next byte in table
        add     di,8
        pop     si              ; set pointer back to start of temp_char
        cmp     al,128          ; through with the ROM table?
        jne     find_char_loop
;
        push    ds              ; must change destination ptr to user
        xor     di,di           ; pointer at vector location 07ch
        mov     ds,di           ; abs0 segment
assume	ds:abs0
	les	di,gr_char_ext	; user character table pointer
        pop     ds              ; restore ds for source ptr in si
assume	ds:data
        jmp    find_char_loop
;
return_graphics_char_code:
;
        pop     di              ; restore stack
        pop     si
        jmp     video_exit
;
;**********************************************************************
;*
;*      Function 9 - writes both a character and attribute at the
;*      specified cursor position.
;*
;*      On entry:
;*
;*              AL contains the character code to be written,
;*              BH contains the display page number,
;*              BL contains attribute to be written,
;*              CX contains a count of characters to be written.
;*
;*      On exit:
;*
;*              Nothing is returned.
;*
;**********************************************************************
;
write_char_and_attribute:
;*
;*      First determine whether to write in alphanumeric or graphics
;*      mode, since each must be handled differently.
;*
        cmp     video_mode,4            ; graphics modes are 4,5,6
        jb      find_alpha_write_posn
;
        cmp     video_mode,monochrome_mode      ; check for monochrome card
        je      find_alpha_write_posn
;
        jmp     short graphics_mode_write
;
find_alpha_write_posn:
;*
;*      Get the video ram address of the cursor for the specified
;*      display page.  It will be returned in the DI register by
;*      the subroutine call.  Thus the address will be set up for
;*      the stosw instruction using the ES segment register.
;*
        call    compute_cursor_address
        mov     bh,bl           ; save attribute and character in BX
        mov     bl,al
;*
;*      Now look at the display enable bit of the status port to do
;*      the read during a horizontal retrace to avoid interference
;*      with the display.
;*
        mov     dx,video_port_base      ; 03b4h or 03d4h
        add     dx,6                    ; video card status register
;
wait_for_low_dispen:
;
        in      al,dx                   ; wait for a low level
        test    al,display_enable_bit   ; bit 1
        jnz     wait_for_low_dispen
        cli             ; don't want to be interrupted here !
;
wait_for_high_dispen:
;
        in      al,dx                   ; wait for a high level
        test    al,display_enable_bit
        jz      wait_for_high_dispen

;
        mov     ax,bx           ; retrieve the data word
        stosw                   ; write it into video ram
        sti                     ; and turn interrupts back on
;
        loop    wait_for_low_dispen     ; do all characters
;
        jmp     video_exit
;
graphics_mode_write:
;
        push    cx              ; save character count
        call    compute_graphics_cursor_posn    ; dest. address
        push    si              ; save address for later
	mov	dx,ds		; save bios data segment
        test    al,080h         ; rom or user character gen. table?
        jnz     code_in_user_table
;
	mov	si,cs		; this table is in rom
	mov	ds,si
        mov     si,offset graphics_char_table   ; rom table
        jmp    short add_in_char_offset
;
code_in_user_table:
;
        and     al,07fh         ; strip off bit 7
        xor     si,si           ; pointer is in segment 0
        mov     ds,si
assume	ds:abs0
	les	si,gr_char_ext	; pointer to user character set
;
add_in_char_offset:
;
	push	es		; segment for video ram
	push	ds		; segment for character table
        mov     cl,3            ; 8 bytes per character
        xor     ah,ah           ; clear upper byte for shift
        shl     ax,cl           ; divide by 8
        add     si,ax           ; add to base of table
;
check_for_hi_res_mode:
;
	mov	ds,dx		; restore bios data segment
assume	ds:data
        cmp     video_mode,hi_res_mode
        je      hi_res_char_write
;*
;*      Here a 320x200 graphics character is to be written.
;*
	pop	es		; graphics character table segment
        mov     dh,bl           ; get the color attribute specified
        and     dh,03h          ; strip off the garbage
        mov     di,offset temp_char     ; use a temporary array
        xor     ax,ax           ; temporary byte
        mov     dl,al           ; byte counter
;
expand_byte_loop:
;
	mov	ch,es:[si]	; get next byte from the char. table
        mov     cl,8            ; bit counter
;
expand_the_byte:
;
        shl     ax,1
        shl     ax,1            ; 2 bits per pixel
        shl     ch,1            ; next bit background or foreground?
        jc      add_in_attribute
;
        and     al,0feh         ; background - clear the two bits
        jmp    short do_another_bit
;
add_in_attribute:
;
        or      al,dh
;
do_another_bit:
;
        loop    expand_the_byte
;
        inc     si              ; next byte from character table
        inc     dl              ; bump the byte counter
        mov     [di],ah         ; store first 4 pixels in temp array
        inc     di
        mov     [di],al         ; store second 4 pixels in next byte
        inc     di
        cmp     dl,8            ; all 8 bytes expanded to 2 bits/pixel
        jb      expand_byte_loop
;
        mov     si,offset temp_char     ; now update the pointer
	pop	es		; segment back to video ram
        pop     di              ; get the video ram pointer back
        pop     cx              ; restore character counter
;
write_a_med_res_char:
;
        xor     dl,dl           ; word counter to 0
        push    si
        push    di              ; save pointers for next character
;
med_res_write_loop:
;
        mov     ax,[si]         ; get the next word to write
        test    dl,1            ; even=even scan row, odd=odd scan row
        jz      med_wr_even_scan
;
        test    bl,080h         ; xor the character?
        jnz     xor_med_char_in
;
        mov     es:02000h[di],ax        ; do the next odd scan word
        add     di,80                   ; bump to next even/odd scan
        jmp    short bump_write_ptrs
;
xor_med_char_in:
;
        xor     es:02000h[di],ax
        add     di,80                   ; bump to next even/odd scan
        jmp    short bump_write_ptrs
;
med_wr_even_scan:
;
        test    bl,080h         ; xor the character?
        jnz     xor_in_med_char
;
        mov     es:[di],ax
        jmp    short bump_write_ptrs
;
xor_in_med_char:
;
        xor     es:[di],ax
;
bump_write_ptrs:
;
        add     si,2            ; next word in temp_char array
        inc     dl              ; bump the word counter
        cmp     dl,8            ; all done?
        jb      med_res_write_loop
;
        pop     di              ; restore pointers for next char
        pop     si
        add     di,2            ; next character column
        loop    write_a_med_res_char
        jmp     video_exit
;*
;*      Here a 640x200 character is to be written at the cursor posn.
;*
hi_res_char_write:
;
	pop	ds		; graphics character segment
	pop	es		; video ram segment
        pop     di              ; get the video ram pointer
        pop     cx              ; restore the character counter
;
write_a_hi_res_char:
;
        xor     dl,dl           ; byte counter
        push    si
        push    di              ; save pointers for next character
;
hi_res_write_char_loop:
;
assume	ds:cseg
	mov	al,[si]		; get the next byte to write
        test    dl,1            ; even=even scan row, odd=odd scan row
        jz      even_scan_hi_res_wr
;
        test    bl,080h         ; xor the character in?
        jnz     xor_hi_res_odd_scan
;
        mov     es:02000h[di],al
        add     di,80           ; bump to next even/odd scan
        jmp    short advance_write_ptrs
;
xor_hi_res_odd_scan:
;
        xor     es:02000h[di],al
        add     di,80           ; bump to next even/odd scan
        jmp    short advance_write_ptrs
;
even_scan_hi_res_wr:
;
        test    bl,080h         ; xor in the character?
        jnz     xor_hi_res_even_scan
;
        mov     es:[di],al
        jmp    short advance_write_ptrs
;
xor_hi_res_even_scan:
;
        xor     es:[di],al
;
advance_write_ptrs:
;
        inc     si              ; next byte in temp_char array
        inc     dl              ; bump the byte counter
        cmp     dl,8            ; all done?
        jb      hi_res_write_char_loop
;
        pop     di
        pop     si
        inc     di              ; point to next character column
        loop    write_a_hi_res_char
;
assume	ds:data
        jmp     video_exit
;
;**********************************************************************
;*
;*      Function 10 - writes a new character at the cursor position
;*      of the specified display page, but does not alter the attibute.
;*
;*      On entry:
;*
;*              AL contains the character code to be written,
;*              BH contains the display page number,
;*              CX contains a count of characters to be written.
;*
;*      On exit:
;*
;*              Nothing is returned.
;*
;**********************************************************************
;
write_character_only:
;*
;*      First determine whether to write in alphanumeric or graphics
;*      mode, since each must be handled differently.
;*
        cmp     video_mode,4            ; graphics modes are 4,5,6
        jb      find_write_conly_posn
;
        cmp     video_mode,monochrome_mode      ; check for monochrome card
        je      find_write_conly_posn
;
        jmp     graphics_mode_write     ; same as for function 9
;
find_write_conly_posn:
;*
;*      Get the video ram address of the cursor for the specified
;*      display page.  It will be returned in the DI register by
;*      the subroutine call.  Thus the address will be set up for
;*      the stosb instruction using the ES segment register.
;*
        call    compute_cursor_address
        mov     bl,al           ; save the character code

;*
;*      Now look at the display enable bit of the status port to do
;*      the read during a horizontal retrace to avoid interference
;*      with the display.
;*
        mov     dx,video_port_base      ; 03b4h or 03d4h
        add     dx,6                    ; video card status register
;
wait_for_low_dspenb:
;
        in      al,dx                   ; wait for a low level
        test    al,display_enable_bit   ; bit 1
        jnz     wait_for_low_dspenb
        cli             ; don't want to be interrupted here !
;
wait_for_high_dspenb:
;
        in      al,dx                   ; wait for a high level
        test    al,display_enable_bit
        jz      wait_for_high_dspenb

;
        mov     al,bl           ; retrieve the data byte
        stosb                   ; write it into video ram
        sti                     ; and turn interrupts back on
;
        inc     di              ; skip the attribute byte
        loop    wait_for_low_dspenb     ; do all characters
;
        jmp     video_exit
;*
;**********************************************************************
;*
;*      Function 11 - selects a new color pallette.
;*
;**********************************************************************
;*
set_color_pallette:
        mov     dx,video_port_base
        add     dx,5            ; 3d9h is the color select port
        mov     al,current_color_pallette       ; get current setting
        cmp     bh,0
        je      set_new_background
;
        and     al,0dfh         ; strip off the pallette bit
        test    bl,1            ; should only be 0 or 1
        je      output_new_color
;
        or      al,020h         ; 1 so set the pallette bit to 1
        jmp    short output_new_color
;
set_new_background:
;
        and     al,0e0h         ; strip off old color information
	and	bl,01fh
        or      al,bl           ; add new color information
;
output_new_color:
;
        out     dx,al           ; write it to port xd9h
        mov     current_color_pallette,al       ; and save it
        jmp     video_exit
;*
;**********************************************************************
;*
;*      Function 12 - write a single pixel element,
;*      Function 13 - read a single pixel element.
;*
;*      Note:  both routines share a common entry point.
;*
;**********************************************************************
;*
read_or_write_a_pixel:
        push    ax              ; save function code
        xor     si,si           ; use SI to point to byte offset
        test    dl,1            ; odd or even scan row?
        jz      compute_row_offset
;
        mov     si,02000h       ; odd scan row here
;
compute_row_offset:
;
        and     dx,0feh         ; strip off odd/even row bit
        mov     al,040          ; dx is already multiplied by 2
        mul     dl              ; 80 bytes per row on the display
        add     si,ax           ; add row offset to memory pointer
        push    cx              ; save column value for later use
;*
;*      Have to compute different values for 320x200 and 640x200 modes.
;*
        cmp     video_mode,hi_res_mode
        je      hi_res_graphics
;
        mov     bl,0c0h         ; initial med res bit mask value
        and     cl,03h          ; determine which bit in the byte
        shl     cl,1            ; *2 since 2 bits per pixel
        mov     ch,cl           ; and save the result
        mov     dl,6            ; determine number of bits to shift
        sub     dl,cl           ; to right justify the pixel data
        mov     dh,2            ; number of bits per pixel
        mov     bh,2            ; number of shifts for byte offset
        jmp    short compute_col_offset
;
hi_res_graphics:
;
        mov     bl,080h         ; initial high res bit mask value
        and     cl,07h          ; strip off all but bit location
        mov     ch,cl           ; and save the result
        mov     dl,07h          ; determine number of bits to shift
        sub     dl,cl           ; to right to justify pixel data
        mov     dh,1            ; number of bits per pixel
        mov     bh,3            ; number of shifts for byte offset
;
compute_col_offset:
;
        pop     ax              ; recover column value
        mov     cl,bh           ; set up shift counter
        shr     ax,cl           ; and determine the byte location
        add     si,ax           ; then add it to the offset in si
        mov     cl,ch           ; set up the shift counter
        shr     bl,cl           ; shift bit mask to correct location
;*
;*      Now look at the function code to determine if a read or write
;*      is to be performed.
;*
        pop     ax              ; recover the function code
        cmp     ah,write_a_pixel
        je      write_a_dot
;
        mov     al,es:[si]      ; read the byte containing the pixel
        and     al,bl           ; mask off the other bits
        mov     cl,dl           ; set up the shift counter
        shr     ax,cl           ; right justify the data
        jmp     video_exit
;*
;*      Here the function is to write a pixel into the specified
;*      location.  If bit 7 in the data byte is set, then the new
;*      pixel value is to be exclusive or'ed into the display memory.
;*
write_a_dot:
;
        mov     ah,al           ; save the original pixel data
        mov     cl,dl           ; set up the shift counter
        shl     al,cl           ; correct the position of the data
        mov     dl,es:[si]      ; read in the current byte
        test    ah,080h         ; check for exclusive or command
        jnz     xor_the_pixel
;
        not     bl              ; complement the mask byte
        and     dl,bl           ; strip off current pixel bits
        or      dl,al           ; write the new pixel information
;
        jmp    short write_new_pixel
;
xor_the_pixel:
;
        xor     dl,al           ; do the exclusive or
;
write_new_pixel:
;
        mov     es:[si],dl      ; write the new data back in ram
        jmp     video_exit
;*
;**********************************************************************
;*
;*      Function 14 - emulates TTY output on the display screen of the
;*                    active display page.  Cursor positioning is taken
;*                    care of internally and the CR, LF, BS, and BEL
;*                    characters are treated as commands.
;*
;*      On entry:
;*
;*              AL contains the character to be written to the display,
;*              BL contains a foreground color for graphics modes.
;*
;*      On exit:
;*
;*              Nothing is returned, character will have been added to
;*              the display area.
;*
;**********************************************************************
;*
emulate_tty_output:
;*
;*	First set up the display page to use, and then compute the
;*	cursor position for it.  The monochrome adapter can only use
;*	page 0.  The current active display page will be used for the
;*	color adapter card.
;*
	cmp	video_mode,monochrome		; only use page 0
	jne	get_color_page
;
	xor	bh,bh
	jmp	short tty1
;
get_color_page:
;
	mov	bh,active_display_page		; color adpater here
;
tty1:   push    bx      ; save page number
        mov     bl,bh   ; then use it as an offset in the cursor table
        xor     bh,bh
        shl     bx,1    ; *2 for word offsets
        mov     si,bx   ; use the si register now and later on
        mov     dx,cursor_position_table[si]    ; get cursor position
        pop     bx      ; restore page and foreground color
;*
;*      Now check for the special characters: CR, LF, BS, BEL
;*
        cmp     al,cr           ; carriage return?
        jne     check_if_lf
;
        xor     dl,dl           ; yes, set column back to zero
        jmp    short tty_exit
;
check_if_lf:
;
        cmp     al,lf           ; line feed?
        jne     check_if_bs
;
        cmp     dh,display_lines ; yes, check if at bottom of screen
        jne     set_new_line    ; and do a scroll if so
;
        call    tty_scroll
        jmp    short tty_exit
;
set_new_line:
;
        inc     dh              ; increment row number
        jmp    short tty_exit
;
check_if_bs:
;
        cmp     al,backspace    ; backspace?
        jne     check_if_bel
;
        cmp     dl,0            ; already at column 0?
        je      tty_exit        ; do nothing if so
;
        dec     dl              ; otherwise decrement column number
        jmp    short tty_exit
;
check_if_bel:
;
        cmp     al,bell         ; bell character?
        jne     write_tty_char
;
	push	ax
	push	bx
	push	dx
	mov	ax,1000		; 1000 hertz tone
	mov	bx,10		; tone duration
	xor	dl,dl		; use speaker channel 0
	mov	dh,3		; use timed duration mode
	call	play_tone
	pop	dx
	pop	bx
	pop	ax
        jmp    short video_exit
;
write_tty_char:
;
        mov     ah,010          ; write char only, BH contains page#
        mov     cx,1            ; do one character only
        int     010h            ; use the interrupt routine
;*
;*      Now update the cursor position.
;*
	inc	dl		; next column
        cmp     dl,byte ptr number_of_display_columns   ; end of line?
        jne    tty_exit
;
check_if_at_bottom:
;
        mov     dl,0            ; yes check for bottom of display
        cmp     dh,display_lines
        jne     no_scroll_needed
;
        call    tty_scroll
        jmp    short tty_exit
;
no_scroll_needed:
;
        inc     dh
;
tty_exit:
;
        mov     cursor_position_table[si],dx    ; update cursor
        call    move_the_cursor
;
        jmp     short video_exit
;
;**********************************************************************
;*
;*      Function 15 - returns the current video state of the system.
;*
;*      On entry:
;*
;*              No parameters passed.
;*
;*      On exit:
;*
;*              AL contains the current video mode set by fn #0,
;*              AH contains the number of display columns,
;*              BH contains the current active display page number.
;*
;**********************************************************************
;
return_video_state:
;*
;*      Since the original BX register is stored on the stack, the data
;*      for the active display page will be placed there so it will be
;*      correct when the registers are restored by the exit code for
;*      int 010h software interrupt handler.
;*
        mov     ah,byte ptr number_of_display_columns
        mov     al,video_mode
	push	bp
	mov	bp,sp
        mov     bx,6[bp]        ; get the original BX register value
        mov     bh,active_display_page
        mov     6[bp],bx        ; write the new BX register value
	pop	bp
;
;        jmp     video_exit	; not needed here
;
;**********************************************************************
;*
;*      This is the exit point for all function calls via the int 010h
;*      software interrupt handler.
;*
;**********************************************************************
;
video_exit:
;
        pop     dx
        pop     cx
        pop     bx
        pop     di
        pop     si
        pop     es
        pop     ds
;
        iret
;
vid_int_hdlr	endp
;
;**********************************************************************
;*
;*      The following set of subroutines are used to support the
;*      function calls of the int 010h software interrupt handler.
;*
;**********************************************************************
;*
;*      Subroutine name - output_cx_to_6845
;*
;*      Purpose - to output the two bytes of the CX register, high byte
;*                followed by low byte, into two consecutive registers
;*                of the 6845 CRT controller.
;*
;*      On entry:
;*
;*              CX contains the two data bytes,
;*              AH contains the offset of the first 6845 register,
;*              Variable video_port_base contains the i/o port addr.
;*
;*      On exit:
;*
;*              Nothing is returned, AX and DX are destroyed
;*
;**********************************************************************
;
output_cx_to_6845	proc
;
;*
;*      First get the i/o port address, then output the register index
;*      in AH followed by the high byte of CX.
;*
        mov     dx,video_port_base      ; setup the port
        mov     al,ah                   ; get the register index
        out     dx,al
;
        inc     dx                      ; point to 6845 data register
        mov     al,ch                   ; get high byte of data
        out     dx,al
;*
;*      Now output the data byte in CL to the next 6845 register.
;*
        dec     dx                      ; point back to 6845 index
        inc     ah                      ; increment to next register
        mov     al,ah
        out     dx,al
;
        inc     dx                      ; point to 6845 data register
        mov     al,cl
        out     dx,al
;
        ret             ; done !
;
output_cx_to_6845	endp
;
;**********************************************************************
;*
;*      Subroutine name - update_cursor_position
;*
;*      Purpose - to update the cursor position for the specified page
;*                and to move the cursor if page is the active display
;*                page.
;*
;*      On entry:
;*
;*              DX contains the new row (DH) and column (DL),
;*              BH contains the display page to be updated.
;*
;*      On exit:
;*
;*              Nothing is returned, cursor is moved on the display if
;*              the active display page was specified.
;*
;**********************************************************************
;*
update_cursor_position	proc
;*
;*      First update the cursor position table for the page.
;*
        push    bx
        mov     bl,bh           ; put page into low byte for table
        xor     bh,bh           ; offset
        shl     bx,1            ; *2 for word offsets
        mov     cursor_position_table[bx],dx    ; updata cursor
;*
;*      Now check if the cursor is in the active display page.  If it
;*      is then the cursor position must be moved in the display.
;*
        shr     bx,1            ; restore page number
        cmp     active_display_page,bl  ; active page ?
        jne     set_cursor_exit
;
        call    move_the_cursor         ; move it to new position
;
set_cursor_exit:
;
        pop     bx
        ret
;
update_cursor_position	endp
;*
;**********************************************************************
;*
;*      Subroutine name - move_cursor
;*
;*      Purpose - to move the cursor in the active display page to a
;*                new postion specified by the DX register.
;*
;*      On entry:
;*
;*              DX contains the new position, row in DH, column in DL
;*
;*      On exit:
;*
;*              Nothing is returned, AX,CX,DX and DI are destroyed.
;*
;**********************************************************************
;
move_the_cursor	proc
;*
;*      First get the offset in the display area, and add to the
;*      starting address of the display page.  The position is
;*      passed in the DX register, and the offset is returned in
;*      the DI register.
;*
        call    compute_video_ram_addr
        add     di,display_start_address
;*
;*      Address is in DI, output it to the 6845 CRT controller
;*
        mov     cx,di           ; set up for the call
        shr     cx,1            ; divide by 2 for char count
        mov     ah,cursor_posn_reg
        call    output_cx_to_6845
;
        ret             ; cursor has been moved !
;
move_the_cursor	endp
;
;**********************************************************************
;*
;*      Subroutine name - compute_cursor_address
;*
;*      Purpose - to compute and return the video ram location of the
;*      cursor position of the display page specified in BH.
;*
;*      On entry:
;*
;*              BH contains the display page number.
;*
;*      On exit:
;*
;*              DI contains the video ram address of the cursor for the
;*              specified display page,
;*              DX is destroyed.
;*
;**********************************************************************
;
compute_cursor_address	proc
;*
;*      First get the cursor position for the display page and put it
;*      in DX for the subroutine call.
;*
        push    ax
        push    bx
        mov     ax,display_ram_size     ; need to compute the starting
        mov     bl,bh                   ; address of the page
        xor     bh,bh                   ; set up for table offset
        mul     bx                      ; page# x display size
        shl     bx,1                    ; *2 for word offsets
        add     ax,cursor_position_table[bx]    ; add cursor position
        mov     dx,ax                   ; to starting address
        call    compute_video_ram_addr
;
        pop     bx
        pop     ax
;
        ret
;
compute_cursor_address	endp
;
;**********************************************************************
;*
;*      Subroutine name - compute_video_ram_addr
;*
;*      Purpose - to compute the video ram offset of the row/column
;*                position passed in the DX register.
;*
;*      On entry:
;*
;*              DX contains the position, row in DH, column in DL,
;*              Variable number_of_display_columns contains the number
;*              of columns per row in the display.
;*
;*      On exit:
;*
;*              DI contains an offset relative to the starting location
;*              of the position passed in DX,
;*              AX and DX are destroyed.
;*
;**********************************************************************
;*
compute_video_ram_addr	proc
;*
;*      The offset from the starting location in the video ram is:
;*
;*      (((row number)(number of columns per row) + column number)*2)
;*
        xor     ah,ah
        mov     al,dh           ; row number
        mul     byte ptr number_of_display_columns
        xor     dh,dh
        add     ax,dx           ; (row#)(#cols per row) + col#
        shl     ax,1            ; multiply by 2 for char + attrib
        mov     di,ax           ; return the computed value in DI
        ret
;
compute_video_ram_addr	endp
;*
;**********************************************************************
;*
;*	Scroll_handler scrolls the display for both alphanumeric and
;*	graphics character modes.
;*
;**********************************************************************
;*
scroll_handler	proc
;*
;*      First determine whether this is an alpha or graphics mode scroll
;*
        cmp     video_mode,4            ; graphics modes are 4,5,6
	jae	check_for_bw
        jmp     scroll_alpha
;
check_for_bw:
;
        cmp     video_mode,monochrome_mode
	jne	scroll_graphics
	jmp	scroll_alpha
;
scroll_graphics:
;
	push	ax		; save row count
	mov	ax,cx
	call	compute_gr_pos	; get offset of upper left corner
	mov	di,si		; and save it
	mov	ax,dx
	call	compute_gr_pos	; get offset of lower right corner
	add	si,240		; correct for 4 scans per char
	mov	dx,320		; 80 bytes * 4 scans per char.
	pop	ax		; get row count back
	xor	ah,ah		; strip off function code
	push	ax		; save row count for later use
	mul	dx		; #rows * #words per row * 4 scans
	mov	dx,ax		; save the result
	mov	cx,si		; compute total word count of scroll
	sub	cx,di		; area
	shr	cx,1		; convert to a word count
	cmp	bl,0		; scroll up or down ?
	jne	scroll_gr_down
;
scroll_gr_up:
;
	mov	si,di		; fix pointers
	add	si,dx
	jmp	short move_gr_rows
;
scroll_gr_down:
;
	mov	di,si		; fix pointers
	sub	si,dx
;
move_gr_rows:
;
	pop	ax		; get row count back
	push	es		; set up segments for scroll
	pop	ds
	cmp	al,0		; clear entire scroll area ?
	je	clear_gr_rows
;
	shr	dx,1		; convert to a word count
	push	si		; save pointers for odd scan
	push	di
	sub	cx,dx		; correct the count value and then
	push	cx		; save count for odd scan
	rep	movsw
	pop	cx
	pop	di
	pop	si
	add	di,02000h	; now do the odd scan rows
	add	si,02000h
	rep	movsw
	mov	cx,dx		; count value for the clear
;
clear_gr_rows:
;
	mov	al,bh
	mov	ah,al		; set up the fill attribute
	push	si		; save pointers for even scan
	push	di
	push	cx
	rep	stosw
	pop	cx
	pop	di
	pop	si
	sub	di,02000h	; now do the even scan rows
	sub	si,02000h
	rep	stosw
;
	ret
;
;*	The following subroutine computes the offset in the video
;*	ram of the specified corner (in the AX register) during a
;*	graphics scroll operation, and returns the result in the
;*	SI register.
;*
compute_gr_pos:
;
	push	ax		; save the row/column positions
	mov	al,ah
	mul	byte ptr number_of_display_columns
	shl	ax,1		; 4 scans per character
	shl	ax,1
	mov	si,ax
	pop	ax		; get original row/col back
	xor	ah,ah		; strip off row information
	add	si,ax		; add in the column number
	cmp	video_mode,hi_res_mode
	je	gr_pos_exit
	shl	si,1		; 2 bytes per char for med res modes
;
gr_pos_exit:
;
	ret
;
scroll_alpha:
;*
;*      Determine the video ram addresses of both corners of the block
;*      to be moved, and the number of data words in the block.
;*
        push    ax              ; save the row count
        call    compute_video_ram_addr  ; row/col in DX
        mov     si,di           ; save offset of lower right corner
        mov     dx,cx           ; now do upper left corner of block
        call    compute_video_ram_addr
;
        mov     ax,si           ; compute the word count
        sub     ax,di
	shr	ax,1		; convert #bytes to # words
	inc	ax		; for 0 origin
	mov	cx,ax		; save for a counter value
;*
;*      Now compute the number of words that will be overwritten as
;*      number of rows to scroll * number of columns in the display.
;*
        pop     ax              ; restore the row count and then
        push    ax              ; save it again for later use
        xor     ah,ah           ; strip off the video function#
        mul     byte ptr number_of_display_columns
        mov     dx,ax           ; save the count for the clear later
	shl	ax,1		; *2 for word offsets later
;*
;*      Here a the pointers are set up for the correct direction of the
;*      scroll.  For a scroll up BL will be 0, and for a scroll down, BL
;*      will be a -1.
;*
        cmp     bl,0
        jne     scroll_alpha_down
;
scroll_alpha_up:
;
        mov     si,di           ; di contains the correct destination
        add     si,ax           ; address, ax contains source offset
        jmp    short check_for_80x25_mode
;
scroll_alpha_down:
;
        mov     di,si           ; si contains the correct destination
        sub     si,ax           ; address, ax contains source offset
;
check_for_80x25_mode:
;
        cmp     video_mode,2    ; turn off color card for 80x25 modes
        jb      check_for_block_clear
;
        cmp     video_mode,3
        ja      check_for_block_clear
;*
;*      Wait for retrace, then turn off the video to generate a
;*      smoother looking scroll.
;*
        push    dx              ; 80x25 color card
        mov     dx,03dah
;
wait_for_retrace:
        in      al,dx
        test    al,8
        jz      wait_for_retrace
;
        mov     dx,03D8h        ; mode register port address
        mov     al,025h         ; disable video - bit 3 low
        out     dx,al
        pop     dx
;
check_for_block_clear:
;
        pop     ax              ; restore row count
        push    ds              ; and set up DS for video ram
        push    es
        pop     ds
;
        cmp     al,0            ; if number of rows=0, clear the
        je      clear_rows      ; whole block - CX contains the count
;
        sub     cx,dx           ; correct the word count for the scroll
        rep     movsw           ; move the block of data
        mov     cx,dx           ; and then clear the rest of the block
;
clear_rows:
;
        mov     ah,bh           ; get the fill attribute
        mov     al,space        ; blank space as the character
        rep     stosw           ; fill with the blanks
;
        pop     ds              ; restore to graphics data segment
        cmp     video_mode,monochrome_mode
        je      scroll_exit     ; done if monochrome card
;
        mov     dx,03D8h        ; otherwise reset the color card
        mov     al,current_control_setting
        out     dx,al
;
scroll_exit:
;
        ret
;
scroll_handler	endp
;*
;**********************************************************************
;*
;*	Scroll from a function 14 call through int 010h
;*
;**********************************************************************
;*
tty_scroll	proc
;*
;*      Check if alpha or graphics mode to determine the proper fill
;*      attribute to use in the scroll.
;*
        push    ax
        push    dx
	push	si
        cmp     video_mode,4            ; graphics modes are 4,5,6
        jb      read_a_char
;
        cmp     video_mode,monochrome_mode      ; b/w card installed?
        je      read_a_char
;
        xor     bh,bh           ; use 0 for background fill
        jmp    short scroll_the_screen
;
read_a_char:
;
        mov     cursor_position_table[si],dx    ; first update the
        call    move_the_cursor                 ; cursor for the read
        mov     ah,8				; BH contains page #
        int     010h
        mov     bh,ah                           ; save the attribute
;
scroll_the_screen:
;
        mov     ax,0601h        ; scroll_up one line
        mov     dh,display_lines ; entire active display area
        mov     dl,byte ptr number_of_display_columns
        dec     dl              ; correct for 0 origin
        xor     cx,cx
        int     010h
;
	pop	si
        pop     dx
        pop     ax
;
        ret
;
tty_scroll	endp
;*
;**********************************************************************
;*
;*      Subroutine name - compute_graphics_cursor_posn
;*
;*      Purpose - to compute and return the video ram location of the
;*      first scan position for the graphics character at the cursor
;*      position which is stored in the first word of the cursor postion
;*      table.
;*
;*      On entry:
;*
;*              Cursor_position_table(0) contains the cursor position.
;*
;*      On exit:
;*
;*              SI contains the offset in video ram,
;*              No registers are destroyed.
;*
;**********************************************************************
;*
compute_graphics_cursor_posn	proc
;
        push    ax
        push    cx
        mov     ax,cursor_position_table
        push    ax              ; save the cursor position
        mov     al,ah           ; get the row number
        mul     byte ptr number_of_display_columns
        mov     cl,2            ; *4 since 4 odd and 4 even scan rows
        shl     ax,cl           ; per character
        mov     si,ax           ; use si to return the value
        pop     ax              ; cursor row/col
        xor     ah,ah           ; now add in the column value
        add     si,ax
;
        cmp     video_mode,hi_res_mode
        je      cgcp
;
        shl     si,1            ; 2 bit per pixel for medium res. mode
;
cgcp:   pop     cx
        pop     ax
        ret
;
compute_graphics_cursor_posn	endp
;
;**********************************************************************
;*
;*	The following routines support the use of the MPX-16 system
;*	with an RS-232 video terminal instead of the monitor/keyboard
;*	configuration.  Note that the INT 010h and INT 016h software
;*	interrupts are still used for console i/o.
;*
;**********************************************************************
;
serial_out_init	proc	near
;
	push	ax
	push	bx
	push	cx
	push	dx
	mov	dx,0161h		; 8251A control register
	xor	al,al
	out	dx,al			; initialize chip
	out	dx,al
	out	dx,al
	mov	al,040h
	out	dx,al
	mov	al,0cah			; 2 stop, no parity, 7 data, x16
	out	dx,al
	mov	al,037h			; enable uart chip
	out	dx,al
;*
;*	Now set up the baudrate frequency
;*
	mov	dx,01A0h		; read dip switches
	in	al,dx
	mov	cl,4
	shr	al,cl			; use as table offset
	and	ax,0eh
	mov	bx,offset baud_rate	; use rom bios table
	add	bx,ax
	mov	bx,cs:[bx]		; get counter value
	mov	al,036h			; 8253 control byte
	mov	dx,01E3h		; 8253 control port address
	out	dx,al
	mov	dx,01E0h		; console timer port
	mov	al,bl			; do lsb first
	out	dx,al
	mov	al,bh			; then do msb
	out	dx,al
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	ret
;
serial_out_init	endp
;
serial_con_output proc	near
;
	push	ax			; save the data byte
	mov	dx,0161h		; 8251a control register
	xor	cx,cx			; timeout value
;
get_serial_status:
;
	in	al,dx
	test	al,1			; ready to xmit ?
	jnz	xmit_the_char
	loop	get_serial_status
	pop	ax			; just return if timed out
	ret
;
xmit_the_char:
;
	pop	ax			; get the data byte
	dec	dx			; 8251 data port
	out	dx,al
	ret
;
serial_con_output endp
;
ser_con_out	proc	far
	iret			; implement buffered i/o later
ser_con_out	endp
;
;
;**********************************************************************
;*
;*      Video_jump_table contains offsets of the functions which are
;*      accessed via an int 010h software interrupt call.  The offsets
;*      in this table are used in an indirect jump to the specified
;*      routine.
;*
;**********************************************************************
;
video_jump_table        dw      offset set_video_mode
                        dw      offset set_cursor_type
                        dw      offset set_cursor_position
                        dw      offset read_cursor_position
                        dw      offset read_light_pen_position
                        dw      offset select_display_page
                        dw      offset scroll_page_up
                        dw      offset scroll_page_down
                        dw      offset read_char_and_attribute
                        dw      offset write_char_and_attribute
                        dw      offset write_character_only
                        dw      offset set_color_pallette
                        dw      offset read_or_write_a_pixel
                        dw      offset read_or_write_a_pixel
                        dw      offset emulate_tty_output
                        dw      offset return_video_state
;
;
;**********************************************************************
;*
;*      The following data tables are used during initialization of the
;*      monochrome or color adapter card during a function 0 call via
;*      the int 010h software interrupt.  The first set of tables are
;*      used to initialize the 6845 CRT controller chips for each of
;*      the standard operating modes.
;*
;**********************************************************************
;*
;*      Video initialization tables:
;*
;*      These four tables are used to initialize the 6845 registers.
;*      The bytes in the tables coorespond directly to the 6845
;*      registers as follows:
;*
;*      r0      horizontal chars total
;*      r1      horizontal chars displayed
;*      r2      hsync position
;*      r3      hsync width
;*      r4      vertical chars total
;*      r5      vtotal adjust
;*      r6      vertical chars displayed
;*      r7      vsync position
;*      r8      interlace mode
;*      r9      maximum scan line address
;*      r10     cursor starting scan line
;*      r11     cursor ending scan line
;*      r12     start address - high byte
;*      r13     start address - low byte
;*      r14     cursor address - high byte
;*      r15     cursor address - low byte
;*      r16     light pen address - high byte
;*      r17     light pen address - low byte
;*
;**********************************************************************
;
;*
;*      40x25 mode for color card
;*
vid_parm_table	label	byte
public	vid_parm_table
        db      038h,028h,02dh,0ah,01fh,6,019h,01ch,2,7,6,7,0,0,0,0
;
video_parm_length equ   offset $ - offset vid_parm_table
;*
;*      80x25 mode for color card
;*
        db      071h,050h,05ah,0ah,01fh,6,019h,01ch,2,7,6,7,0,0,0,0
;*
;*      graphics modes for color card
;*
        db      038h,028h,02dh,0ah,07fh,6,064h,070h,2,1,6,7,0,0,0,0
;*
;*      80x25 monochrome adapter card
;*
        db      061h,050h,052h,0fh,019h,6,019h,019h,2,0dh,0bh,0ch
        db      0,0,0,0
;
;**********************************************************************
;*
;*      The video control table is used to select the proper setting
;*      for the monochrome or color card control port at 03b8h or at
;*      03d8h, respectively.
;*
;**********************************************************************
;
video_control_table     db      02ch,028h,02dh,029h,02ah,02eh,01eh,029h
;
;**********************************************************************
;*
;*      The columns table is used to initialize the system for the
;*      correct number of display columns for each mode.
;*
;**********************************************************************
;
columns_table   db      40,40,80,80,40,40,80,80
;
;**********************************************************************
;*
;*      The display length table is used to initialize the system for
;*      the correct number of video ram words used in the active
;*      display area.
;*
;**********************************************************************
;
display_length_table    dw              2048            ; 40x25 modes
                        dw              4096            ; 80x25 modes
                        dw              16384           ; graphics modes
                        dw              16384           ; graphics modes
;
cseg	ends
	end
